@@ -271,7 +271,7 @@ impl HrtfRender {
 
                         // Convert to Right Handed, this is what HRTF crate expects
                         let new_sample_vector = Vec3 {
-                            z: new_sample_vector.z * -1.0,
+                            z: -new_sample_vector.z,
                             ..new_sample_vector
                         };
 
@@ -476,16 +476,16 @@ impl ObjectImpl for HrtfRender {
 
                 let mut state_guard = self.state.lock().unwrap();
 
-                if let Some(state) = state_guard.as_mut() {
-                    if objs.len() != state.ininfo.channels() as usize {
-                        gst::warning!(
-                            CAT,
-                            "Could not update spatial objects, expected {} channels, got {}",
-                            state.ininfo.channels(),
-                            objs.len()
-                        );
-                        return;
-                    }
+                if let Some(state) = state_guard.as_mut()
+                    && objs.len() != state.ininfo.channels() as usize
+                {
+                    gst::warning!(
+                        CAT,
+                        "Could not update spatial objects, expected {} channels, got {}",
+                        state.ininfo.channels(),
+                        objs.len()
+                    );
+                    return;
                 }
 
                 settings.spatial_objects = if objs.is_empty() { None } else { Some(objs) };
@@ -782,20 +782,23 @@ impl BaseTransformImpl for HrtfRender {
         let mut thread_pool_g = THREAD_POOL.lock().unwrap();
         let mut thread_pool = self.thread_pool.lock().unwrap();
 
-        if let Some(tp) = thread_pool_g.upgrade() {
-            *thread_pool = Some(tp);
-        } else {
-            let tp = ThreadPoolBuilder::new().build().map_err(|_| {
-                gst::error_msg!(
-                    gst::CoreError::Failed,
-                    ["Could not create rayon thread pool"]
-                )
-            })?;
+        match thread_pool_g.upgrade() {
+            Some(tp) => {
+                *thread_pool = Some(tp);
+            }
+            _ => {
+                let tp = ThreadPoolBuilder::new().build().map_err(|_| {
+                    gst::error_msg!(
+                        gst::CoreError::Failed,
+                        ["Could not create rayon thread pool"]
+                    )
+                })?;
 
-            let tp = Arc::new(tp);
+                let tp = Arc::new(tp);
 
-            *thread_pool = Some(tp);
-            *thread_pool_g = Arc::downgrade(thread_pool.as_ref().unwrap());
+                *thread_pool = Some(tp);
+                *thread_pool_g = Arc::downgrade(thread_pool.as_ref().unwrap());
+            }
         }
 
         Ok(())
