@@ -1,22 +1,11 @@
 // Copyright (C) 2018 Sebastian Dröge <sebastian@centricular.com>
 // Copyright (C) 2019-2022 François Laignel <fengalin@free.fr>
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This Source Code Form is subject to the terms of the Mozilla Public License, v2.0.
+// If a copy of the MPL was not distributed with this file, You can obtain one at
+// <https://mozilla.org/MPL/2.0/>.
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the
-// Free Software Foundation, Inc., 51 Franklin Street, Suite 500,
-// Boston, MA 02110-1335, USA.
-//
-// SPDX-License-Identifier: LGPL-2.1-or-later
+// SPDX-License-Identifier: MPL-2.0
 
 use futures::channel::mpsc;
 use futures::prelude::*;
@@ -85,8 +74,10 @@ impl PadSrcHandler for AppSrcPadHandler {
 
         use gst::EventView;
         let ret = match event.view() {
-            EventView::FlushStart(..) => imp.task.flush_start().await_maybe_on_context().is_ok(),
-            EventView::FlushStop(..) => imp.task.flush_stop().await_maybe_on_context().is_ok(),
+            EventView::FlushStart(..) => {
+                imp.task.flush_start().block_on_or_add_subtask(pad).is_ok()
+            }
+            EventView::FlushStop(..) => imp.task.flush_stop().block_on_or_add_subtask(pad).is_ok(),
             EventView::Reconfigure(..) => true,
             EventView::Latency(..) => true,
             _ => false,
@@ -225,6 +216,10 @@ impl AppSrcTask {
 impl TaskImpl for AppSrcTask {
     type Item = StreamItem;
 
+    fn obj(&self) -> &impl IsA<glib::Object> {
+        &self.element
+    }
+
     async fn try_next(&mut self) -> Result<StreamItem, gst::FlowError> {
         self.receiver
             .next()
@@ -302,16 +297,19 @@ impl AppSrc {
         let do_timestamp = self.settings.lock().unwrap().do_timestamp;
         if do_timestamp {
             let elem = self.obj();
-            if let Some(clock) = elem.clock() {
-                let base_time = elem.base_time();
-                let now = clock.time();
+            match elem.clock() {
+                Some(clock) => {
+                    let base_time = elem.base_time();
+                    let now = clock.time();
 
-                let buffer = buffer.make_mut();
-                buffer.set_dts(now.opt_checked_sub(base_time).ok().flatten());
-                buffer.set_pts(None);
-            } else {
-                gst::error!(CAT, imp = self, "Don't have a clock yet");
-                return false;
+                    let buffer = buffer.make_mut();
+                    buffer.set_dts(now.opt_checked_sub(base_time).ok().flatten());
+                    buffer.set_pts(None);
+                }
+                _ => {
+                    gst::error!(CAT, imp = self, "Don't have a clock yet");
+                    return false;
+                }
             }
         }
 
@@ -368,37 +366,47 @@ impl AppSrc {
 
         let (sender, receiver) = mpsc::channel(max_buffers);
         *self.sender.lock().unwrap() = Some(sender);
-
         self.task
             .prepare(AppSrcTask::new(self.obj().clone(), receiver), context)
-            .block_on()?;
-
-        gst::debug!(CAT, imp = self, "Prepared");
-
-        Ok(())
+            .block_on_or_add_subtask_then(self.obj(), |elem, res| {
+                if res.is_ok() {
+                    gst::debug!(CAT, obj = elem, "Prepared");
+                }
+            })
     }
 
     fn unprepare(&self) {
         gst::debug!(CAT, imp = self, "Unpreparing");
 
         *self.sender.lock().unwrap() = None;
-        self.task.unprepare().block_on().unwrap();
-
-        gst::debug!(CAT, imp = self, "Unprepared");
+        let _ = self
+            .task
+            .unprepare()
+            .block_on_or_add_subtask_then(self.obj(), |elem, _| {
+                gst::debug!(CAT, obj = elem, "Unprepared");
+            });
     }
 
     fn stop(&self) -> Result<(), gst::ErrorMessage> {
         gst::debug!(CAT, imp = self, "Stopping");
-        self.task.stop().block_on()?;
-        gst::debug!(CAT, imp = self, "Stopped");
-        Ok(())
+        self.task
+            .stop()
+            .block_on_or_add_subtask_then(self.obj(), |elem, res| {
+                if res.is_ok() {
+                    gst::debug!(CAT, obj = elem, "Stopped");
+                }
+            })
     }
 
     fn start(&self) -> Result<(), gst::ErrorMessage> {
         gst::debug!(CAT, imp = self, "Starting");
-        self.task.start().block_on()?;
-        gst::debug!(CAT, imp = self, "Started");
-        Ok(())
+        self.task
+            .start()
+            .block_on_or_add_subtask_then(self.obj(), |elem, res| {
+                if res.is_ok() {
+                    gst::debug!(CAT, obj = elem, "Started");
+                }
+            })
     }
 }
 
