@@ -1,21 +1,10 @@
 // Copyright (C) 2018 Sebastian Dröge <sebastian@centricular.com>
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This Source Code Form is subject to the terms of the Mozilla Public License, v2.0.
+// If a copy of the MPL was not distributed with this file, You can obtain one at
+// <https://mozilla.org/MPL/2.0/>.
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the
-// Free Software Foundation, Inc., 51 Franklin Street, Suite 500,
-// Boston, MA 02110-1335, USA.
-//
-// SPDX-License-Identifier: LGPL-2.1-or-later
+// SPDX-License-Identifier: MPL-2.0
 
 use futures::channel::oneshot;
 
@@ -23,11 +12,8 @@ use gst::glib;
 use gst::prelude::*;
 use gst::subclass::prelude::*;
 
-use std::sync::LazyLock;
-
 use std::collections::{HashMap, VecDeque};
-use std::sync::{Arc, Weak};
-use std::sync::{Mutex, MutexGuard};
+use std::sync::{Arc, LazyLock, Mutex, MutexGuard, Weak};
 use std::time::Duration;
 
 use crate::runtime::prelude::*;
@@ -107,8 +93,9 @@ struct ProxyContextInner {
     dataqueue: Option<DataQueue>,
     last_res: Result<gst::FlowSuccess, gst::FlowError>,
     pending_queue: Option<PendingQueue>,
-    have_sink: bool,
-    have_src: bool,
+    upstream_latency: Option<gst::ClockTime>,
+    sink: Option<super::ProxySink>,
+    src: Option<super::ProxySrc>,
 }
 
 impl Drop for ProxyContextInner {
@@ -119,49 +106,35 @@ impl Drop for ProxyContextInner {
 }
 
 #[derive(Debug)]
-struct ProxyContext {
+struct ProxyContextSink {
     shared: Arc<Mutex<ProxyContextInner>>,
-    as_sink: bool,
     name: String,
 }
 
-impl ProxyContext {
+impl ProxyContextSink {
     #[inline]
     fn lock_shared(&self) -> MutexGuard<'_, ProxyContextInner> {
         self.shared.lock().unwrap()
     }
 
-    fn get(name: &str, as_sink: bool) -> Option<Self> {
+    fn add(name: &str, sink: &super::ProxySink) -> Option<Self> {
         let mut proxy_ctxs = PROXY_CONTEXTS.lock().unwrap();
 
         let mut proxy_ctx = None;
-        if let Some(shared_weak) = proxy_ctxs.get(name) {
-            if let Some(shared) = shared_weak.upgrade() {
-                {
-                    let shared = shared.lock().unwrap();
-                    if (shared.have_sink && as_sink) || (shared.have_src && !as_sink) {
-                        return None;
-                    }
+        if let Some(shared_weak) = proxy_ctxs.get(name)
+            && let Some(shared) = shared_weak.upgrade()
+        {
+            {
+                let shared = shared.lock().unwrap();
+                if shared.sink.is_some() {
+                    return None;
                 }
-
-                proxy_ctx = Some({
-                    let proxy_ctx = ProxyContext {
-                        shared,
-                        as_sink,
-                        name: name.into(),
-                    };
-                    {
-                        let mut shared = proxy_ctx.lock_shared();
-                        if as_sink {
-                            shared.have_sink = true;
-                        } else {
-                            shared.have_src = true;
-                        }
-                    }
-
-                    proxy_ctx
-                });
             }
+
+            proxy_ctx = Some(ProxyContextSink {
+                shared,
+                name: name.into(),
+            });
         }
 
         if proxy_ctx.is_none() {
@@ -170,15 +143,15 @@ impl ProxyContext {
                 dataqueue: None,
                 last_res: Err(gst::FlowError::Flushing),
                 pending_queue: None,
-                have_sink: as_sink,
-                have_src: !as_sink,
+                upstream_latency: None,
+                sink: Some(sink.clone()),
+                src: None,
             }));
 
             proxy_ctxs.insert(name.into(), Arc::downgrade(&shared));
 
-            proxy_ctx = Some(ProxyContext {
+            proxy_ctx = Some(ProxyContextSink {
                 shared,
-                as_sink,
                 name: name.into(),
             });
         }
@@ -187,18 +160,74 @@ impl ProxyContext {
     }
 }
 
-impl Drop for ProxyContext {
+impl Drop for ProxyContextSink {
     fn drop(&mut self) {
         let mut shared_ctx = self.lock_shared();
-        if self.as_sink {
-            assert!(shared_ctx.have_sink);
-            shared_ctx.have_sink = false;
-            let _ = shared_ctx.pending_queue.take();
-        } else {
-            assert!(shared_ctx.have_src);
-            shared_ctx.have_src = false;
-            let _ = shared_ctx.dataqueue.take();
+        shared_ctx.sink = None;
+        let _ = shared_ctx.pending_queue.take();
+    }
+}
+
+#[derive(Debug)]
+struct ProxyContextSrc {
+    shared: Arc<Mutex<ProxyContextInner>>,
+    name: String,
+}
+
+impl ProxyContextSrc {
+    #[inline]
+    fn lock_shared(&self) -> MutexGuard<'_, ProxyContextInner> {
+        self.shared.lock().unwrap()
+    }
+
+    fn add(name: &str, src: &super::ProxySrc) -> Option<Self> {
+        let mut proxy_ctxs = PROXY_CONTEXTS.lock().unwrap();
+
+        let mut proxy_ctx = None;
+        if let Some(shared_weak) = proxy_ctxs.get(name)
+            && let Some(shared) = shared_weak.upgrade()
+        {
+            {
+                let shared = shared.lock().unwrap();
+                if shared.src.is_some() {
+                    return None;
+                }
+            }
+
+            proxy_ctx = Some(ProxyContextSrc {
+                shared,
+                name: name.into(),
+            });
         }
+
+        if proxy_ctx.is_none() {
+            let shared = Arc::new(Mutex::new(ProxyContextInner {
+                name: name.into(),
+                dataqueue: None,
+                last_res: Err(gst::FlowError::Flushing),
+                pending_queue: None,
+                upstream_latency: None,
+                sink: None,
+                src: Some(src.clone()),
+            }));
+
+            proxy_ctxs.insert(name.into(), Arc::downgrade(&shared));
+
+            proxy_ctx = Some(ProxyContextSrc {
+                shared,
+                name: name.into(),
+            });
+        }
+
+        proxy_ctx
+    }
+}
+
+impl Drop for ProxyContextSrc {
+    fn drop(&mut self) {
+        let mut shared_ctx = self.lock_shared();
+        shared_ctx.src = None;
+        let _ = shared_ctx.dataqueue.take();
     }
 }
 
@@ -289,7 +318,7 @@ impl PadSinkHandler for ProxySinkPadHandler {
 #[derive(Debug)]
 pub struct ProxySink {
     sink_pad: PadSink,
-    proxy_ctx: Mutex<Option<ProxyContext>>,
+    proxy_ctx: Mutex<Option<ProxyContextSink>>,
     settings: Mutex<SettingsSink>,
 }
 
@@ -302,6 +331,18 @@ static SINK_CAT: LazyLock<gst::DebugCategory> = LazyLock::new(|| {
 });
 
 impl ProxySink {
+    fn push(&self, dataqueue: &DataQueue, item: DataQueueItem) -> Result<(), DataQueueItem> {
+        if dataqueue
+            .push(self.obj().upcast_ref(), item)?
+            .is_first_buffer()
+        {
+            let elem = self.obj();
+            let _ = elem.post_message(gst::message::Latency::builder().src(&*elem).build());
+        }
+
+        Ok(())
+    }
+
     async fn schedule_pending_queue(&self) {
         loop {
             let more_queue_space_receiver = {
@@ -317,10 +358,10 @@ impl ProxySink {
                 } = *shared_ctx;
 
                 if let Some(ref mut pending_queue) = *pq {
-                    if let Some(ref dataqueue) = dataqueue {
+                    if let Some(dataqueue) = dataqueue {
                         let mut failed_item = None;
                         while let Some(item) = pending_queue.items.pop_front() {
-                            if let Err(item) = dataqueue.push(item) {
+                            if let Err(item) = self.push(dataqueue, item) {
                                 failed_item = Some(item);
                                 break;
                             }
@@ -372,12 +413,12 @@ impl ProxySink {
                 } = *shared_ctx;
 
                 match (pending_queue, dataqueue) {
-                    (None, Some(ref dataqueue)) => dataqueue.push(item),
-                    (Some(ref mut pending_queue), Some(ref dataqueue)) => {
+                    (None, Some(dataqueue)) => self.push(dataqueue, item),
+                    (Some(pending_queue), Some(dataqueue)) => {
                         if !pending_queue.scheduled {
                             let mut failed_item = None;
                             while let Some(item) = pending_queue.items.pop_front() {
-                                if let Err(item) = dataqueue.push(item) {
+                                if let Err(item) = self.push(dataqueue, item) {
                                     failed_item = Some(item);
                                     break;
                                 }
@@ -388,7 +429,7 @@ impl ProxySink {
 
                                 Err(item)
                             } else {
-                                dataqueue.push(item)
+                                self.push(dataqueue, item)
                             }
                         } else {
                             Err(item)
@@ -406,6 +447,10 @@ impl ProxySink {
                     .unwrap_or(true)
                 {
                     if shared_ctx.pending_queue.is_none() {
+                        if shared_ctx.last_res == Err(gst::FlowError::Flushing) {
+                            return Err(gst::FlowError::Flushing);
+                        }
+
                         shared_ctx.pending_queue = Some(PendingQueue::default());
                     }
 
@@ -465,7 +510,7 @@ impl ProxySink {
 
         let proxy_context = self.settings.lock().unwrap().proxy_context.to_string();
 
-        let proxy_ctx = ProxyContext::get(&proxy_context, true).ok_or_else(|| {
+        let proxy_ctx = ProxyContextSink::add(&proxy_context, &self.obj()).ok_or_else(|| {
             gst::error_msg!(
                 gst::ResourceError::OpenRead,
                 ["Failed to create or get ProxyContext"]
@@ -542,11 +587,13 @@ impl ObjectSubclass for ProxySink {
 impl ObjectImpl for ProxySink {
     fn properties() -> &'static [glib::ParamSpec] {
         static PROPERTIES: LazyLock<Vec<glib::ParamSpec>> = LazyLock::new(|| {
-            vec![glib::ParamSpecString::builder("proxy-context")
-                .nick("Proxy Context")
-                .blurb("Context name of the proxy to share with")
-                .default_value(Some(DEFAULT_PROXY_CONTEXT))
-                .build()]
+            vec![
+                glib::ParamSpecString::builder("proxy-context")
+                    .nick("Proxy Context")
+                    .blurb("Context name of the proxy to share with")
+                    .default_value(Some(DEFAULT_PROXY_CONTEXT))
+                    .build(),
+            ]
         });
 
         PROPERTIES.as_ref()
@@ -598,6 +645,27 @@ impl ElementImpl for ProxySink {
         Some(&*ELEMENT_METADATA)
     }
 
+    fn send_event(&self, event: gst::Event) -> bool {
+        gst::log!(SINK_CAT, imp = self, "Got {event:?}");
+
+        if let gst::EventView::Latency(lat_evt) = event.view() {
+            let latency = lat_evt.latency();
+
+            let proxy_ctx = self.proxy_ctx.lock().unwrap();
+            let mut shared_ctx = proxy_ctx.as_ref().unwrap().lock_shared();
+            shared_ctx.upstream_latency = Some(latency);
+
+            if let Some(src) = shared_ctx.src.as_ref() {
+                gst::log!(SINK_CAT, imp = self, "Setting upstream latency {latency}");
+                src.imp().set_upstream_latency(latency);
+            } else {
+                gst::info!(SINK_CAT, imp = self, "No sources to set upstream latency");
+            }
+        }
+
+        self.sink_pad.gst_pad().push_event(event)
+    }
+
     fn pad_templates() -> &'static [gst::PadTemplate] {
         static PAD_TEMPLATES: LazyLock<Vec<gst::PadTemplate>> = LazyLock::new(|| {
             let caps = gst::Caps::new_any();
@@ -671,28 +739,36 @@ impl PadSrcHandler for ProxySrcPadHandler {
         use gst::EventView;
         match event.view() {
             EventView::FlushStart(..) => {
-                if let Err(err) = imp.task.flush_start().await_maybe_on_context() {
-                    gst::error!(SRC_CAT, obj = pad, "FlushStart failed {:?}", err);
-                    gst::element_imp_error!(
-                        imp,
-                        gst::StreamError::Failed,
-                        ("Internal data stream error"),
-                        ["FlushStart failed {:?}", err]
-                    );
-                    return false;
-                }
+                let _ =
+                    imp.task
+                        .flush_start()
+                        .block_on_or_add_subtask_then(imp.obj(), |elem, res| {
+                            if let Err(err) = res {
+                                gst::error!(SRC_CAT, obj = elem, "FlushStart failed {err:?}");
+                                gst::element_error!(
+                                    elem,
+                                    gst::StreamError::Failed,
+                                    ("Internal data stream error"),
+                                    ["FlushStart failed {err:?}"]
+                                );
+                            }
+                        });
             }
             EventView::FlushStop(..) => {
-                if let Err(err) = imp.task.flush_stop().await_maybe_on_context() {
-                    gst::error!(SRC_CAT, obj = pad, "FlushStop failed {:?}", err);
-                    gst::element_imp_error!(
-                        imp,
-                        gst::StreamError::Failed,
-                        ("Internal data stream error"),
-                        ["FlushStop failed {:?}", err]
-                    );
-                    return false;
-                }
+                let _ =
+                    imp.task
+                        .flush_stop()
+                        .block_on_or_add_subtask_then(imp.obj(), |elem, res| {
+                            if let Err(err) = res {
+                                gst::error!(SRC_CAT, obj = elem, "FlushStop failed {err:?}");
+                                gst::element_error!(
+                                    elem,
+                                    gst::StreamError::Failed,
+                                    ("Internal data stream error"),
+                                    ["FlushStop failed {err:?}"]
+                                );
+                            }
+                        });
             }
             _ => (),
         }
@@ -706,13 +782,44 @@ impl PadSrcHandler for ProxySrcPadHandler {
         }
     }
 
-    fn src_query(self, pad: &gst::Pad, _proxysrc: &ProxySrc, query: &mut gst::QueryRef) -> bool {
+    fn src_query(self, pad: &gst::Pad, proxysrc: &ProxySrc, query: &mut gst::QueryRef) -> bool {
         gst::log!(SRC_CAT, obj = pad, "Handling {:?}", query);
 
         use gst::QueryViewMut;
         let ret = match query.view_mut() {
             QueryViewMut::Latency(q) => {
-                q.set(true, gst::ClockTime::ZERO, gst::ClockTime::NONE);
+                let Some(mut min) = *proxysrc.upstream_latency.lock().unwrap() else {
+                    gst::debug!(
+                        SRC_CAT,
+                        obj = pad,
+                        "Upstream latency not available yet, can't handle {query:?}"
+                    );
+                    return false;
+                };
+
+                let upstream_ctx = proxysrc
+                    .dataqueue
+                    .lock()
+                    .unwrap()
+                    .as_ref()
+                    .and_then(|dq| dq.upstream_context());
+                let is_same_ts_ctx = upstream_ctx == *proxysrc.ts_ctx.lock().unwrap();
+
+                let settings = proxysrc.settings.lock().unwrap();
+                if !is_same_ts_ctx {
+                    min += gst::ClockTime::from_nseconds(settings.context_wait.as_nanos() as u64);
+                }
+                // TODO also use max-size-buffers & CAPS when applicable
+                let max = settings.max_size_time;
+                drop(settings);
+
+                gst::debug!(
+                    SRC_CAT,
+                    obj = pad,
+                    "Returning latency: live true, min {min}, max {max}"
+                );
+                q.set(true, min, max);
+
                 true
             }
             QueryViewMut::Scheduling(q) => {
@@ -721,14 +828,15 @@ impl PadSrcHandler for ProxySrcPadHandler {
                 true
             }
             QueryViewMut::Caps(q) => {
-                let caps = if let Some(ref caps) = pad.current_caps() {
-                    q.filter()
+                let caps = match pad.current_caps() {
+                    Some(ref caps) => q
+                        .filter()
                         .map(|f| f.intersect_with_mode(caps, gst::CapsIntersectMode::First))
-                        .unwrap_or_else(|| caps.clone())
-                } else {
-                    q.filter()
+                        .unwrap_or_else(|| caps.clone()),
+                    _ => q
+                        .filter()
                         .map(|f| f.to_owned())
-                        .unwrap_or_else(gst::Caps::new_any)
+                        .unwrap_or_else(gst::Caps::new_any),
                 };
 
                 q.set_result(&caps);
@@ -759,12 +867,38 @@ impl ProxySrcTask {
         ProxySrcTask { element, dataqueue }
     }
 
-    async fn push_item(&self, item: DataQueueItem) -> Result<(), gst::FlowError> {
+    /// Tries to get the upstream latency.
+    ///
+    /// This is needed when a `ts-proxysrc` joins the `proxy-context` after
+    /// the matching `ts-proxysink` has already started streaming.
+    async fn maybe_get_upstream_latency(&self) -> Result<(), gst::FlowError> {
+        let proxysrc = self.element.imp();
+
+        if proxysrc.upstream_latency.lock().unwrap().is_some() {
+            return Ok(());
+        }
+
+        gst::log!(SRC_CAT, imp = proxysrc, "Getting upstream latency");
+
+        let proxy_ctx = proxysrc.proxy_ctx.lock().unwrap();
+        let shared_ctx = proxy_ctx.as_ref().unwrap().lock_shared();
+
+        if let Some(latency) = shared_ctx.upstream_latency {
+            proxysrc.set_upstream_latency_priv(latency);
+        } else {
+            gst::log!(SRC_CAT, imp = proxysrc, "Upstream latency is still unknown");
+        }
+
+        Ok(())
+    }
+
+    async fn push_item(&mut self, item: DataQueueItem) -> Result<(), gst::FlowError> {
         let proxysrc = self.element.imp();
 
         {
             let proxy_ctx = proxysrc.proxy_ctx.lock().unwrap();
             let mut shared_ctx = proxy_ctx.as_ref().unwrap().lock_shared();
+
             if let Some(pending_queue) = shared_ctx.pending_queue.as_mut() {
                 pending_queue.notify_more_queue_space();
             }
@@ -772,16 +906,25 @@ impl ProxySrcTask {
 
         match item {
             DataQueueItem::Buffer(buffer) => {
+                self.maybe_get_upstream_latency().await?;
                 gst::log!(SRC_CAT, obj = self.element, "Forwarding {:?}", buffer);
                 proxysrc.src_pad.push(buffer).await.map(drop)
             }
             DataQueueItem::BufferList(list) => {
+                self.maybe_get_upstream_latency().await?;
                 gst::log!(SRC_CAT, obj = self.element, "Forwarding {:?}", list);
                 proxysrc.src_pad.push_list(list).await.map(drop)
             }
             DataQueueItem::Event(event) => {
                 gst::log!(SRC_CAT, obj = self.element, "Forwarding {:?}", event);
+
+                let is_eos = event.type_() == gst::EventType::Eos;
                 proxysrc.src_pad.push_event(event).await;
+
+                if is_eos {
+                    return Err(gst::FlowError::Eos);
+                }
+
                 Ok(())
             }
         }
@@ -791,6 +934,10 @@ impl ProxySrcTask {
 impl TaskImpl for ProxySrcTask {
     type Item = DataQueueItem;
 
+    fn obj(&self) -> &impl IsA<glib::Object> {
+        &self.element
+    }
+
     async fn start(&mut self) -> Result<(), gst::ErrorMessage> {
         gst::log!(SRC_CAT, obj = self.element, "Starting task");
 
@@ -798,14 +945,14 @@ impl TaskImpl for ProxySrcTask {
         let proxy_ctx = proxysrc.proxy_ctx.lock().unwrap();
         let mut shared_ctx = proxy_ctx.as_ref().unwrap().lock_shared();
 
-        shared_ctx.last_res = Ok(gst::FlowSuccess::Ok);
-
         if let Some(pending_queue) = shared_ctx.pending_queue.as_mut() {
             pending_queue.notify_more_queue_space();
         }
 
         self.dataqueue.start();
 
+        shared_ctx.last_res = Ok(gst::FlowSuccess::Ok);
+
         gst::log!(SRC_CAT, obj = self.element, "Task started");
         Ok(())
     }
@@ -858,13 +1005,20 @@ impl TaskImpl for ProxySrcTask {
 
     async fn stop(&mut self) -> Result<(), gst::ErrorMessage> {
         gst::log!(SRC_CAT, obj = self.element, "Stopping task");
+        self.flush_start().await?;
+        gst::log!(SRC_CAT, obj = self.element, "Task stopped");
+        Ok(())
+    }
+
+    async fn flush_start(&mut self) -> Result<(), gst::ErrorMessage> {
+        gst::log!(SRC_CAT, obj = self.element, "Task flush start");
 
         let proxysrc = self.element.imp();
         let proxy_ctx = proxysrc.proxy_ctx.lock().unwrap();
         let mut shared_ctx = proxy_ctx.as_ref().unwrap().lock_shared();
 
-        self.dataqueue.clear();
         self.dataqueue.stop();
+        self.dataqueue.clear();
 
         shared_ctx.last_res = Err(gst::FlowError::Flushing);
 
@@ -872,22 +1026,14 @@ impl TaskImpl for ProxySrcTask {
             pending_queue.notify_more_queue_space();
         }
 
-        gst::log!(SRC_CAT, obj = self.element, "Task stopped");
+        gst::log!(SRC_CAT, obj = self.element, "Task flush started");
         Ok(())
     }
 
-    async fn flush_start(&mut self) -> Result<(), gst::ErrorMessage> {
-        gst::log!(SRC_CAT, obj = self.element, "Starting task flush");
-
-        let proxysrc = self.element.imp();
-        let proxy_ctx = proxysrc.proxy_ctx.lock().unwrap();
-        let mut shared_ctx = proxy_ctx.as_ref().unwrap().lock_shared();
-
-        self.dataqueue.clear();
-
-        shared_ctx.last_res = Err(gst::FlowError::Flushing);
-
-        gst::log!(SRC_CAT, obj = self.element, "Task flush started");
+    async fn flush_stop(&mut self) -> Result<(), gst::ErrorMessage> {
+        gst::log!(SRC_CAT, obj = self.element, "Task flush stop");
+        self.start().await?;
+        gst::log!(SRC_CAT, obj = self.element, "Task flush stopped");
         Ok(())
     }
 }
@@ -896,8 +1042,10 @@ impl TaskImpl for ProxySrcTask {
 pub struct ProxySrc {
     src_pad: PadSrc,
     task: Task,
-    proxy_ctx: Mutex<Option<ProxyContext>>,
+    proxy_ctx: Mutex<Option<ProxyContextSrc>>,
+    ts_ctx: Mutex<Option<Context>>,
     dataqueue: Mutex<Option<DataQueue>>,
+    upstream_latency: Mutex<Option<gst::ClockTime>>,
     settings: Mutex<SettingsSrc>,
 }
 
@@ -910,17 +1058,56 @@ static SRC_CAT: LazyLock<gst::DebugCategory> = LazyLock::new(|| {
 });
 
 impl ProxySrc {
+    // Sets the upstream latency without blocking the caller.
+    pub fn set_upstream_latency(&self, up_latency: gst::ClockTime) {
+        match *self.ts_ctx.lock().unwrap() {
+            Some(ref ts_ctx) => {
+                let obj = self.obj().clone();
+
+                gst::log!(SRC_CAT, imp = self, "Setting upstream latency async");
+                ts_ctx.spawn(async move {
+                    obj.imp().set_upstream_latency_priv(up_latency);
+                });
+            }
+            _ => {
+                gst::debug!(SRC_CAT, imp = self, "Not ready to handle upstream latency");
+            }
+        }
+    }
+
+    // Sets the upstream latency blocking the caller until it's handled.
+    fn set_upstream_latency_priv(&self, new_latency: gst::ClockTime) {
+        {
+            let mut cur_upstream_latency = self.upstream_latency.lock().unwrap();
+            if let Some(cur_upstream_latency) = *cur_upstream_latency
+                && cur_upstream_latency == new_latency
+            {
+                return;
+            }
+            *cur_upstream_latency = Some(new_latency);
+        }
+
+        gst::debug!(
+            SRC_CAT,
+            imp = self,
+            "Got new upstream latency {new_latency}"
+        );
+
+        self.post_message(gst::message::Latency::builder().src(&*self.obj()).build());
+    }
+
     fn prepare(&self) -> Result<(), gst::ErrorMessage> {
         gst::debug!(SRC_CAT, imp = self, "Preparing");
 
         let settings = self.settings.lock().unwrap().clone();
 
-        let proxy_ctx = ProxyContext::get(&settings.proxy_context, false).ok_or_else(|| {
-            gst::error_msg!(
-                gst::ResourceError::OpenRead,
-                ["Failed to create get shared_state"]
-            )
-        })?;
+        let proxy_ctx =
+            ProxyContextSrc::add(&settings.proxy_context, &self.obj()).ok_or_else(|| {
+                gst::error_msg!(
+                    gst::ResourceError::OpenRead,
+                    ["Failed to create get shared_state"]
+                )
+            })?;
 
         let ts_ctx = Context::acquire(&settings.context, settings.context_wait).map_err(|err| {
             gst::error_msg!(
@@ -928,26 +1115,13 @@ impl ProxySrc {
                 ["Failed to acquire Context: {}", err]
             )
         })?;
+        *self.ts_ctx.lock().unwrap() = Some(ts_ctx.clone());
 
-        let dataqueue = DataQueue::new(
-            &self.obj().clone().upcast(),
-            self.src_pad.gst_pad(),
-            if settings.max_size_buffers == 0 {
-                None
-            } else {
-                Some(settings.max_size_buffers)
-            },
-            if settings.max_size_bytes == 0 {
-                None
-            } else {
-                Some(settings.max_size_bytes)
-            },
-            if settings.max_size_time.is_zero() {
-                None
-            } else {
-                Some(settings.max_size_time)
-            },
-        );
+        let dataqueue = DataQueue::builder(self.obj().upcast_ref(), self.src_pad.gst_pad())
+            .max_size_buffers(settings.max_size_buffers)
+            .max_size_bytes(settings.max_size_bytes)
+            .max_size_time(settings.max_size_time)
+            .build();
 
         {
             let mut shared_ctx = proxy_ctx.lock_shared();
@@ -963,11 +1137,11 @@ impl ProxySrc {
 
         self.task
             .prepare(ProxySrcTask::new(self.obj().clone(), dataqueue), ts_ctx)
-            .block_on()?;
-
-        gst::debug!(SRC_CAT, imp = self, "Prepared");
-
-        Ok(())
+            .block_on_or_add_subtask_then(self.obj(), |elem, res| {
+                if res.is_ok() {
+                    gst::debug!(SRC_CAT, obj = elem, "Prepared");
+                }
+            })
     }
 
     fn unprepare(&self) {
@@ -979,33 +1153,39 @@ impl ProxySrc {
             proxy_src_pads.remove(&settings.proxy_context);
         }
 
-        self.task.unprepare().block_on().unwrap();
+        let _ = self
+            .task
+            .unprepare()
+            .block_on_or_add_subtask_then(self.obj(), |elem, _| {
+                let imp = elem.imp();
+                *imp.dataqueue.lock().unwrap() = None;
+                *imp.proxy_ctx.lock().unwrap() = None;
+                *imp.ts_ctx.lock().unwrap() = None;
 
-        *self.dataqueue.lock().unwrap() = None;
-        *self.proxy_ctx.lock().unwrap() = None;
-
-        gst::debug!(SRC_CAT, imp = self, "Unprepared");
+                gst::debug!(SRC_CAT, obj = elem, "Unprepared");
+            });
     }
 
     fn stop(&self) -> Result<(), gst::ErrorMessage> {
         gst::debug!(SRC_CAT, imp = self, "Stopping");
-        self.task.stop().await_maybe_on_context()?;
-        gst::debug!(SRC_CAT, imp = self, "Stopped");
-        Ok(())
+        self.task
+            .stop()
+            .block_on_or_add_subtask_then(self.obj(), |elem, res| {
+                if res.is_ok() {
+                    gst::debug!(SRC_CAT, obj = elem, "Stopped");
+                }
+            })
     }
 
     fn start(&self) -> Result<(), gst::ErrorMessage> {
         gst::debug!(SRC_CAT, imp = self, "Starting");
-        self.task.start().await_maybe_on_context()?;
-        gst::debug!(SRC_CAT, imp = self, "Started");
-        Ok(())
-    }
-
-    fn pause(&self) -> Result<(), gst::ErrorMessage> {
-        gst::debug!(SRC_CAT, imp = self, "Pausing");
-        self.task.pause().block_on()?;
-        gst::debug!(SRC_CAT, imp = self, "Paused");
-        Ok(())
+        self.task
+            .start()
+            .block_on_or_add_subtask_then(self.obj(), |elem, res| {
+                if res.is_ok() {
+                    gst::debug!(SRC_CAT, obj = elem, "Started");
+                }
+            })
     }
 }
 
@@ -1023,7 +1203,9 @@ impl ObjectSubclass for ProxySrc {
             ),
             task: Task::default(),
             proxy_ctx: Mutex::new(None),
+            ts_ctx: Mutex::new(None),
             dataqueue: Mutex::new(None),
+            upstream_latency: Mutex::new(None),
             settings: Mutex::new(SettingsSrc::default()),
         }
     }
@@ -1065,6 +1247,21 @@ impl ObjectImpl for ProxySrc {
                     .maximum(u64::MAX - 1)
                     .default_value(DEFAULT_MAX_SIZE_TIME.nseconds())
                     .build(),
+                glib::ParamSpecUInt::builder("current-level-buffers")
+                    .nick("Current Level Buffers")
+                    .blurb("Current number of buffers in the queue")
+                    .read_only()
+                    .build(),
+                glib::ParamSpecUInt::builder("current-level-bytes")
+                    .nick("Current Level Bytes")
+                    .blurb("Current amount of data in the queue (bytes)")
+                    .read_only()
+                    .build(),
+                glib::ParamSpecUInt64::builder("current-level-time")
+                    .nick("Current Level Time")
+                    .blurb("Current amount of data in the queue (in ns)")
+                    .read_only()
+                    .build(),
             ]
         });
 
@@ -1105,14 +1302,43 @@ impl ObjectImpl for ProxySrc {
     }
 
     fn property(&self, _id: usize, pspec: &glib::ParamSpec) -> glib::Value {
-        let settings = self.settings.lock().unwrap();
         match pspec.name() {
-            "max-size-buffers" => settings.max_size_buffers.to_value(),
-            "max-size-bytes" => settings.max_size_bytes.to_value(),
-            "max-size-time" => settings.max_size_time.nseconds().to_value(),
-            "context" => settings.context.to_value(),
-            "context-wait" => (settings.context_wait.as_millis() as u32).to_value(),
-            "proxy-context" => settings.proxy_context.to_value(),
+            "max-size-buffers" => self.settings.lock().unwrap().max_size_buffers.to_value(),
+            "max-size-bytes" => self.settings.lock().unwrap().max_size_bytes.to_value(),
+            "max-size-time" => self
+                .settings
+                .lock()
+                .unwrap()
+                .max_size_time
+                .nseconds()
+                .to_value(),
+            "current-level-buffers" => self
+                .dataqueue
+                .lock()
+                .unwrap()
+                .as_ref()
+                .map_or(0, |d| d.cur_level_buffers())
+                .to_value(),
+            "current-level-bytes" => self
+                .dataqueue
+                .lock()
+                .unwrap()
+                .as_ref()
+                .map_or(0, |d| d.cur_level_bytes())
+                .to_value(),
+            "current-level-time" => self
+                .dataqueue
+                .lock()
+                .unwrap()
+                .as_ref()
+                .map_or(gst::ClockTime::ZERO, |d| d.cur_level_time())
+                .nseconds()
+                .to_value(),
+            "context" => self.settings.lock().unwrap().context.to_value(),
+            "context-wait" => {
+                (self.settings.lock().unwrap().context_wait.as_millis() as u32).to_value()
+            }
+            "proxy-context" => self.settings.lock().unwrap().proxy_context.to_value(),
             _ => unimplemented!(),
         }
     }
@@ -1173,8 +1399,8 @@ impl ElementImpl for ProxySrc {
                     gst::StateChangeError
                 })?;
             }
-            gst::StateChange::PlayingToPaused => {
-                self.pause().map_err(|_| gst::StateChangeError)?;
+            gst::StateChange::PausedToReady => {
+                self.stop().map_err(|_| gst::StateChangeError)?;
             }
             gst::StateChange::ReadyToNull => {
                 self.unprepare();
@@ -1194,9 +1420,6 @@ impl ElementImpl for ProxySrc {
             gst::StateChange::PlayingToPaused => {
                 success = gst::StateChangeSuccess::NoPreroll;
             }
-            gst::StateChange::PausedToReady => {
-                self.stop().map_err(|_| gst::StateChangeError)?;
-            }
             _ => (),
         }
 
