@@ -85,7 +85,7 @@ use std::sync::{LazyLock, Mutex};
 use std::time::Duration;
 
 use crate::runtime::prelude::*;
-use crate::runtime::{self, timer, PadSrc, Task};
+use crate::runtime::{self, PadSrc, Task, timer};
 
 static CAT: LazyLock<gst::DebugCategory> = LazyLock::new(|| {
     gst::DebugCategory::new(
@@ -410,6 +410,10 @@ enum TaskItem {
 impl TaskImpl for RTPDTMFSrcTask {
     type Item = TaskItem;
 
+    fn obj(&self) -> &impl IsA<glib::Object> {
+        &self.elem
+    }
+
     async fn start(&mut self) -> Result<(), gst::ErrorMessage> {
         gst::log!(CAT, obj = self.elem, "Starting Task");
 
@@ -802,29 +806,32 @@ impl RTPDTMFSrcTask {
                     }
                 }
 
-                if let Ok(Some(ptime)) = s.get_optional::<u32>("ptime") {
-                    gst::log!(CAT, imp = imp, "Using peer ptime {ptime}");
-                    self.ptime = gst::ClockTime::from_mseconds(ptime as u64);
-                    *imp.ptime.lock().unwrap() = self.ptime;
-                } else {
-                    match s.get_optional::<u32>("maxptime") {
-                        Ok(Some(maxptime)) => {
-                            gst::log!(CAT, imp = imp, "Using peer maxptime {maxptime}");
-                            self.ptime = gst::ClockTime::from_mseconds(maxptime as u64);
-                            *imp.ptime.lock().unwrap() = self.ptime;
-                        }
-                        other => {
-                            if let Err(err) = other {
-                                // Would be cool to be able to fixate uint
-                                gst::warning!(
-                                    CAT,
-                                    imp = imp,
-                                    "Invalid type for peer 'ptime' / 'maxptime' in {s}: {err}"
-                                );
-                                s.remove_field("maxptime");
+                match s.get_optional::<u32>("ptime") {
+                    Ok(Some(ptime)) => {
+                        gst::log!(CAT, imp = imp, "Using peer ptime {ptime}");
+                        self.ptime = gst::ClockTime::from_mseconds(ptime as u64);
+                        *imp.ptime.lock().unwrap() = self.ptime;
+                    }
+                    _ => {
+                        match s.get_optional::<u32>("maxptime") {
+                            Ok(Some(maxptime)) => {
+                                gst::log!(CAT, imp = imp, "Using peer maxptime {maxptime}");
+                                self.ptime = gst::ClockTime::from_mseconds(maxptime as u64);
+                                *imp.ptime.lock().unwrap() = self.ptime;
+                            }
+                            other => {
+                                if let Err(err) = other {
+                                    // Would be cool to be able to fixate uint
+                                    gst::warning!(
+                                        CAT,
+                                        imp = imp,
+                                        "Invalid type for peer 'ptime' / 'maxptime' in {s}: {err}"
+                                    );
+                                    s.remove_field("maxptime");
+                                }
+                                s.set("ptime", self.ptime);
+                                gst::log!(CAT, imp = imp, "Using internal ptime {}", self.ptime);
                             }
-                            s.set("ptime", self.ptime);
-                            gst::log!(CAT, imp = imp, "Using internal ptime {}", self.ptime);
                         }
                     }
                 }
@@ -1064,49 +1071,60 @@ impl RTPDTMFSrc {
         drop(settings);
 
         let (dtmf_evt_tx, dtmf_evt_rx) = mpsc::channel(DEFAULT_DTMF_EVT_CHAN_CAPACITY);
+        *self.dtmf_evt_tx.lock().unwrap() = Some(dtmf_evt_tx);
         self.task
             .prepare(
                 RTPDTMFSrcTask::new(self.obj().clone(), dtmf_evt_rx),
                 context,
             )
-            .block_on()?;
-        *self.dtmf_evt_tx.lock().unwrap() = Some(dtmf_evt_tx);
-
-        gst::debug!(CAT, imp = self, "Prepared");
-
-        Ok(())
+            .block_on_or_add_subtask_then(self.obj(), |elem, res| {
+                if res.is_ok() {
+                    gst::debug!(CAT, obj = elem, "Prepared");
+                }
+            })
     }
 
     fn unprepare(&self) {
         gst::debug!(CAT, imp = self, "Unpreparing");
 
-        self.task.unprepare().block_on().unwrap();
-        *self.dtmf_evt_tx.lock().unwrap() = None;
+        let _ = self
+            .task
+            .unprepare()
+            .block_on_or_add_subtask_then(self.obj(), |elem, _| {
+                *elem.imp().dtmf_evt_tx.lock().unwrap() = None;
 
-        gst::debug!(CAT, imp = self, "Unprepared");
+                gst::debug!(CAT, obj = elem, "Unprepared");
+            });
     }
 
     fn stop(&self) -> Result<(), gst::ErrorMessage> {
         gst::debug!(CAT, imp = self, "Stopping");
 
-        self.task.stop().block_on()?;
-        {
-            let mut settings = self.settings.lock().unwrap();
-            settings.timestamp = 0;
-            settings.seqnum = 0;
-        }
+        self.task
+            .stop()
+            .block_on_or_add_subtask_then(self.obj(), |elem, res| {
+                {
+                    let imp = elem.imp();
+                    let mut settings = imp.settings.lock().unwrap();
+                    settings.timestamp = 0;
+                    settings.seqnum = 0;
+                }
 
-        gst::debug!(CAT, imp = self, "Stopped");
-
-        Ok(())
+                if res.is_ok() {
+                    gst::debug!(CAT, obj = elem, "Stopped");
+                }
+            })
     }
 
     fn start(&self) -> Result<(), gst::ErrorMessage> {
         gst::debug!(CAT, imp = self, "Starting");
-        self.task.start().block_on()?;
-        gst::debug!(CAT, imp = self, "Started");
-
-        Ok(())
+        self.task
+            .start()
+            .block_on_or_add_subtask_then(self.obj(), |elem, res| {
+                if res.is_ok() {
+                    gst::debug!(CAT, obj = elem, "Started");
+                }
+            })
     }
 }
 
@@ -1164,18 +1182,18 @@ impl ObjectImpl for RTPDTMFSrc {
                     .maximum(u32::MAX)
                     .read_only()
                     .build(),
-                glib::ParamSpecInt::builder("timestamp-offset")
+                glib::ParamSpecInt64::builder("timestamp-offset")
                     .nick("Timestamp Offset")
                     .blurb("Offset to add to all outgoing timestamps (-1 = random)")
                     .minimum(-1)
-                    .maximum(u32::MAX as i32)
-                    .default_value(DEFAULT_TIMESTAMP_OFFSET.map_or(-1i32, |val| val as i32))
+                    .maximum(u32::MAX as i64)
+                    .default_value(DEFAULT_TIMESTAMP_OFFSET.map_or(-1, |val| val as i64))
                     .build(),
                 glib::ParamSpecInt::builder("seqnum-offset")
                     .nick("Sequence Number Offset")
                     .blurb("Offset to add to all outgoing seqnum (-1 => random)")
                     .minimum(-1)
-                    .maximum(i32::MAX)
+                    .maximum(u16::MAX as i32)
                     .default_value(DEFAULT_SEQNUM_OFFSET.map_or(-1i32, |val| val as i32))
                     .build(),
                 glib::ParamSpecUInt::builder("clock-rate")
@@ -1185,12 +1203,12 @@ impl ObjectImpl for RTPDTMFSrc {
                     .maximum(u32::MAX)
                     .default_value(DEFAULT_CLOCK_RATE)
                     .build(),
-                glib::ParamSpecUInt::builder("ssrc")
+                glib::ParamSpecInt64::builder("ssrc")
                     .nick("Synchronization Source (SSRC)")
                     .blurb("The SSRC of the packets (-1 => random)")
-                    .minimum(0)
-                    .maximum(u32::MAX)
-                    .default_value(DEFAULT_SSRC.unwrap_or(u32::MAX))
+                    .minimum(-1)
+                    .maximum(u32::MAX as i64)
+                    .default_value(DEFAULT_SSRC.map_or(-1, |val| val as i64))
                     .build(),
                 glib::ParamSpecUInt::builder("pt")
                     .nick("Payload Type")
@@ -1225,7 +1243,7 @@ impl ObjectImpl for RTPDTMFSrc {
                 settings.context_wait = Duration::from_millis(value.get::<u32>().unwrap().into());
             }
             "timestamp-offset" => {
-                settings.timestamp_offset = value.get::<i32>().unwrap().try_into().ok();
+                settings.timestamp_offset = value.get::<i64>().unwrap().try_into().ok();
             }
             "seqnum-offset" => {
                 settings.seqnum_offset = value.get::<i32>().unwrap().try_into().ok();
@@ -1234,7 +1252,7 @@ impl ObjectImpl for RTPDTMFSrc {
                 settings.clock_rate = value.get::<u32>().unwrap();
             }
             "ssrc" => {
-                settings.ssrc = value.get::<i32>().unwrap().try_into().ok();
+                settings.ssrc = value.get::<i64>().unwrap().try_into().ok();
             }
             "pt" => {
                 settings.pt = value.get::<u32>().unwrap().try_into().unwrap();
@@ -1253,21 +1271,10 @@ impl ObjectImpl for RTPDTMFSrc {
             "context-wait" => (settings.context_wait.as_millis() as u32).to_value(),
             "timestamp" => settings.timestamp.to_value(),
             "seqnum" => settings.seqnum.to_value(),
-            "timestamp-offset" => settings
-                .timestamp_offset
-                .and_then(|val| val.try_into().ok())
-                .unwrap_or(-1i32)
-                .to_value(),
-            "seqnum-offset" => settings
-                .seqnum_offset
-                .map_or(-1i32, |val| val as i32)
-                .to_value(),
+            "timestamp-offset" => settings.timestamp_offset.map_or(-1, i64::from).to_value(),
+            "seqnum-offset" => settings.seqnum_offset.map_or(-1i32, i32::from).to_value(),
             "clock-rate" => settings.clock_rate.to_value(),
-            "ssrc" => settings
-                .ssrc
-                .and_then(|val| val.try_into().ok())
-                .unwrap_or(-1i32)
-                .to_value(),
+            "ssrc" => settings.ssrc.map_or(-1, i64::from).to_value(),
             "pt" => (settings.pt as u32).to_value(),
             "packet-redundancy" => (settings.packet_redundancy as u32).to_value(),
             _ => unimplemented!(),
@@ -1377,8 +1384,8 @@ impl PadSrcHandler for RTPDTMFSrcPadHandler {
 
         use gst::EventView::*;
         match event.view() {
-            FlushStart(..) => imp.task.flush_start().await_maybe_on_context().is_ok(),
-            FlushStop(..) => imp.task.flush_stop().await_maybe_on_context().is_ok(),
+            FlushStart(..) => imp.task.flush_start().block_on_or_add_subtask(pad).is_ok(),
+            FlushStop(..) => imp.task.flush_stop().block_on_or_add_subtask(pad).is_ok(),
             Reconfigure(..) => true,
             Latency(..) => true,
             _ => imp.handle_maybe_dtmf_event(&event),
