@@ -1,22 +1,11 @@
 // Copyright (C) 2018 Sebastian Dröge <sebastian@centricular.com>
 // Copyright (C) 2018 LEE Dongjun <redongjun@gmail.com>
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This Source Code Form is subject to the terms of the Mozilla Public License, v2.0.
+// If a copy of the MPL was not distributed with this file, You can obtain one at
+// <https://mozilla.org/MPL/2.0/>.
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the
-// Free Software Foundation, Inc., 51 Franklin Street, Suite 500,
-// Boston, MA 02110-1335, USA.
-//
-// SPDX-License-Identifier: LGPL-2.1-or-later
+// SPDX-License-Identifier: MPL-2.0
 
 use futures::prelude::*;
 
@@ -37,7 +26,7 @@ use crate::runtime::{Context, PadSrc, Task, TaskState};
 
 use crate::runtime::Async;
 use crate::socket::{Socket, SocketError, SocketRead};
-use futures::channel::mpsc::{channel, Receiver, Sender};
+use futures::channel::mpsc::{Receiver, Sender, channel};
 use futures::pin_mut;
 
 const DEFAULT_HOST: Option<&str> = Some("127.0.0.1");
@@ -105,8 +94,10 @@ impl PadSrcHandler for TcpClientSrcPadHandler {
 
         use gst::EventView;
         let ret = match event.view() {
-            EventView::FlushStart(..) => imp.task.flush_start().await_maybe_on_context().is_ok(),
-            EventView::FlushStop(..) => imp.task.flush_stop().await_maybe_on_context().is_ok(),
+            EventView::FlushStart(..) => {
+                imp.task.flush_start().block_on_or_add_subtask(pad).is_ok()
+            }
+            EventView::FlushStop(..) => imp.task.flush_stop().block_on_or_add_subtask(pad).is_ok(),
             EventView::Reconfigure(..) => true,
             EventView::Latency(..) => true,
             _ => false,
@@ -127,7 +118,10 @@ impl PadSrcHandler for TcpClientSrcPadHandler {
         use gst::QueryViewMut;
         let ret = match query.view_mut() {
             QueryViewMut::Latency(q) => {
-                q.set(false, gst::ClockTime::ZERO, gst::ClockTime::NONE);
+                let latency =
+                    gst::ClockTime::try_from(imp.settings.lock().unwrap().context_wait).unwrap();
+                gst::debug!(CAT, obj = pad, "Reporting latency {latency}");
+                q.set(true, latency, gst::ClockTime::NONE);
                 true
             }
             QueryViewMut::Scheduling(q) => {
@@ -269,6 +263,10 @@ impl TcpClientSrcTask {
 impl TaskImpl for TcpClientSrcTask {
     type Item = gst::Buffer;
 
+    fn obj(&self) -> &impl IsA<glib::Object> {
+        &self.element
+    }
+
     async fn prepare(&mut self) -> Result<(), gst::ErrorMessage> {
         gst::log!(
             CAT,
@@ -514,22 +512,34 @@ impl TcpClientSrc {
 
     fn unprepare(&self) {
         gst::debug!(CAT, imp = self, "Unpreparing");
-        self.task.unprepare().block_on().unwrap();
-        gst::debug!(CAT, imp = self, "Unprepared");
+        let _ = self
+            .task
+            .unprepare()
+            .block_on_or_add_subtask_then(self.obj(), |elem, _| {
+                gst::debug!(CAT, obj = elem, "Unprepared");
+            });
     }
 
     fn stop(&self) -> Result<(), gst::ErrorMessage> {
         gst::debug!(CAT, imp = self, "Stopping");
-        self.task.stop().block_on()?;
-        gst::debug!(CAT, imp = self, "Stopped");
-        Ok(())
+        self.task
+            .stop()
+            .block_on_or_add_subtask_then(self.obj(), |elem, res| {
+                if res.is_ok() {
+                    gst::debug!(CAT, obj = elem, "Stopped");
+                }
+            })
     }
 
     fn start(&self) -> Result<(), gst::ErrorMessage> {
         gst::debug!(CAT, imp = self, "Starting");
-        self.task.start().block_on()?;
-        gst::debug!(CAT, imp = self, "Started");
-        Ok(())
+        self.task
+            .start()
+            .block_on_or_add_subtask_then(self.obj(), |elem, res| {
+                if res.is_ok() {
+                    gst::debug!(CAT, obj = elem, "Started");
+                }
+            })
     }
 
     fn state(&self) -> TaskState {
@@ -731,10 +741,10 @@ impl ElementImpl for TcpClientSrc {
 
         match event.view() {
             EventView::Eos(_) => {
-                if self.state() != TaskState::Started {
-                    if let Err(err) = self.start() {
-                        gst::error!(CAT, imp = self, "Failed to start task thread {err:?}");
-                    }
+                if self.state() != TaskState::Started
+                    && let Err(err) = self.start()
+                {
+                    gst::error!(CAT, imp = self, "Failed to start task thread {err:?}");
                 }
 
                 if self.state() == TaskState::Started {
