@@ -14,10 +14,10 @@ use gst::subclass::prelude::*;
 use gst_base::subclass::prelude::*;
 
 use aws_sdk_s3::{
-    config::{self, retry::RetryConfig, Credentials, Region},
+    Client,
+    config::{self, Credentials, Region, retry::RetryConfig},
     operation::put_object::builders::PutObjectFluentBuilder,
     primitives::ByteStream,
-    Client,
 };
 
 use super::NextFile;
@@ -28,7 +28,7 @@ use std::sync::Mutex;
 use std::time::Duration;
 
 use crate::s3url::*;
-use crate::s3utils::{self, duration_from_millis, duration_to_millis, WaitError};
+use crate::s3utils::{self, WaitError, duration_from_millis, duration_to_millis};
 
 const DEFAULT_RETRY_ATTEMPTS: u32 = 5;
 const DEFAULT_FLUSH_INTERVAL_BUFFERS: u64 = 1;
@@ -171,6 +171,7 @@ impl Default for Settings {
 #[derive(Default)]
 pub struct S3PutObjectSink {
     url: Mutex<Option<GstS3Url>>,
+    s3_uri: Mutex<Option<GstS3Uri>>,
     settings: Mutex<Settings>,
     state: Mutex<State>,
     canceller: Mutex<s3utils::Canceller>,
@@ -193,11 +194,13 @@ impl S3PutObjectSink {
         #[allow(clippy::needless_bool)]
         // Verbose if/else form for readability
         if settings.flush_interval_buffers > 0
-            && (state.num_buffers % settings.flush_interval_buffers) == 0
+            && state
+                .num_buffers
+                .is_multiple_of(settings.flush_interval_buffers)
         {
             true
         } else if settings.flush_interval_bytes > 0
-            && (state.buffer.len() as u64 % settings.flush_interval_bytes) == 0
+            && (state.buffer.len() as u64).is_multiple_of(settings.flush_interval_bytes)
         {
             true
         } else if settings.flush_interval_time.is_some()
@@ -223,18 +226,21 @@ impl S3PutObjectSink {
             unreachable!("Element should be started");
         }
 
-        let s3url = {
-            let url = self.url.lock().unwrap();
-            match *url {
-                Some(ref url) => url.clone(),
-                None => {
-                    return Err(gst::error_msg!(
-                        gst::ResourceError::Settings,
-                        ["Cannot start without a URL being set"]
-                    ));
-                }
-            }
-        };
+        let s3url = self.url.lock().unwrap();
+        let s3uri = self.s3_uri.lock().unwrap();
+        if s3url.is_none() && s3uri.is_none() {
+            return Err(gst::error_msg!(
+                gst::ResourceError::Settings,
+                ["Cannot start without a URL or S3 compatible URI being set"]
+            ));
+        }
+        let region = s3url
+            .as_ref()
+            .map(|u| u.region.clone())
+            .or_else(|| s3uri.as_ref()?.region.clone().map(Region::new));
+        let use_arn_region = s3uri.as_ref().is_some_and(|uri| uri.region.is_some());
+        drop(s3url);
+        drop(s3uri);
 
         let timeout_config = s3utils::timeout_config(settings.request_timeout);
 
@@ -252,23 +258,24 @@ impl S3PutObjectSink {
             _ => None,
         };
 
-        let sdk_config =
-            s3utils::wait_config(&self.canceller, s3url.region.clone(), timeout_config, cred)
-                .map_err(|err| match err {
-                    WaitError::FutureError(err) => gst::error_msg!(
-                        gst::ResourceError::OpenWrite,
-                        ["Failed to create SDK config: {}", err]
-                    ),
-                    WaitError::Cancelled => {
-                        gst::error_msg!(
-                            gst::LibraryError::Failed,
-                            ["SDK config request interrupted during start"]
-                        )
-                    }
-                })?;
+        let sdk_config = s3utils::wait_config(&self.canceller, region, timeout_config, cred)
+            .map_err(|err| match err {
+                WaitError::FutureError(err) => gst::error_msg!(
+                    gst::ResourceError::OpenWrite,
+                    ["Failed to create SDK config: {}", err]
+                ),
+                WaitError::Cancelled => {
+                    gst::error_msg!(
+                        gst::LibraryError::Failed,
+                        ["SDK config request interrupted during start"]
+                    )
+                }
+            })?;
 
         let config_builder = config::Builder::from(&sdk_config)
-            .force_path_style(settings.force_path_style)
+            .use_arn_region(use_arn_region)
+            // Force path style cannot be used with ARN.
+            .force_path_style(settings.force_path_style && !use_arn_region)
             .retry_config(RetryConfig::standard().with_max_attempts(settings.retry_attempts));
 
         let config = if let Some(ref uri) = settings.endpoint_uri {
@@ -311,11 +318,45 @@ impl S3PutObjectSink {
             }
             Err(_) => Err(glib::Error::new(
                 gst::URIError::BadUri,
-                "Could not parse URI",
+                "Could not parse S3 URI",
             )),
         }
     }
 
+    fn set_s3_uri(self: &S3PutObjectSink, uri_str: Option<&str>) -> Result<(), glib::Error> {
+        let state = self.state.lock().unwrap();
+
+        if let State::Started { .. } = *state {
+            return Err(glib::Error::new(
+                gst::URIError::BadState,
+                "Cannot set S3 URI on a started s3src",
+            ));
+        }
+
+        let mut s3_uri = self.s3_uri.lock().unwrap();
+
+        if uri_str.is_none() {
+            *s3_uri = None;
+            return Ok(());
+        }
+
+        let uri_str = uri_str.unwrap();
+        match parse_s3_uri(uri_str) {
+            Ok(s3uri) => {
+                gst::info!(CAT, imp = self, "S3 URI: {s3uri:?}");
+                *s3_uri = Some(s3uri);
+                Ok(())
+            }
+            Err(e) => {
+                gst::error!(CAT, imp = self, "Failed to parse S3 URI: {e}");
+                Err(glib::Error::new(
+                    gst::URIError::BadUri,
+                    "Could not parse S3 URI",
+                ))
+            }
+        }
+    }
+
     fn accumulate_buffer(
         &self,
         buffer: &gst::Buffer,
@@ -363,7 +404,6 @@ impl S3PutObjectSink {
         &self,
         started_state: &mut Started,
     ) -> Result<Option<PutObjectFluentBuilder>, gst::FlowError> {
-        let url = self.url.lock().unwrap();
         let settings = self.settings.lock().unwrap();
 
         if started_state.buffer.is_empty() {
@@ -376,11 +416,10 @@ impl S3PutObjectSink {
             &started_state.buffer,
         ));
 
-        let bucket = Some(url.as_ref().unwrap().bucket.to_owned());
+        let (bucket, object, new_file) = self.get_bucket_and_key();
 
-        let object = url.as_ref().unwrap().object.to_owned();
-        let key = if object.contains("%0") {
-            match sprintf::sprintf!(&object, started_state.index) {
+        let key = if new_file {
+            match sprintf::sprintf!(&object.unwrap().clone(), started_state.index) {
                 Ok(k) => {
                     /* Equivalent to opening a new file */
                     started_state.index += 1;
@@ -398,7 +437,7 @@ impl S3PutObjectSink {
                 }
             }
         } else {
-            Some(object)
+            object
         };
         let metadata = settings.to_metadata(self);
         let client = &started_state.client;
@@ -434,11 +473,13 @@ impl S3PutObjectSink {
             NextFile::MaxDuration => {
                 let mut new_duration = gst::ClockTime::ZERO;
 
-                if buffer.pts().is_some() && started_state.file_start_pts.is_some() {
-                    new_duration = buffer.pts().unwrap() - started_state.file_start_pts.unwrap();
+                if let Some((pts, file_start_pts)) =
+                    Option::zip(buffer.pts(), started_state.file_start_pts)
+                {
+                    new_duration = pts - file_start_pts;
 
-                    if buffer.duration().is_some() {
-                        new_duration += buffer.duration().unwrap();
+                    if let Some(duration) = buffer.duration() {
+                        new_duration += duration;
                     }
                 }
 
@@ -523,6 +564,29 @@ impl S3PutObjectSink {
 
         self.accumulate_buffer(buffer, started_state)
     }
+
+    fn get_bucket_and_key(&self) -> (Option<String>, Option<String>, bool) {
+        let url = self.url.lock().unwrap().clone();
+        let s3_uri = self.s3_uri.lock().unwrap().clone();
+
+        if let Some(s3_uri) = s3_uri {
+            return (
+                Some(s3_uri.bucket.clone()),
+                Some(s3_uri.key.clone()),
+                s3_uri.key.contains("%0"),
+            );
+        }
+
+        if let Some(url) = url {
+            return (
+                Some(url.bucket.clone()),
+                Some(url.object.clone()),
+                url.object.contains("%0"),
+            );
+        }
+
+        unreachable!()
+    }
 }
 
 #[glib::object_subclass]
@@ -649,6 +713,10 @@ impl ObjectImpl for S3PutObjectSink {
                     .blurb("Minimum distance between keyframes to start a new file")
                     .default_value(DEFAULT_MIN_KEYFRAME_DISTANCE.into())
                     .build(),
+                glib::ParamSpecString::builder("s3-uri")
+                    .nick("S3 URI")
+                    .blurb("The S3 compatible URI or S3 Access Point URI to use")
+                    .build(),
             ]
         });
 
@@ -771,6 +839,9 @@ impl ObjectImpl for S3PutObjectSink {
                     .get::<gst::ClockTime>()
                     .expect("type checked upstream");
             }
+            "s3-uri" => {
+                let _ = self.set_s3_uri(value.get().expect("type checked upstream"));
+            }
             _ => unimplemented!(),
         }
     }
@@ -810,6 +881,15 @@ impl ObjectImpl for S3PutObjectSink {
             "force-path-style" => settings.force_path_style.to_value(),
             "min-keyframe-distance" => settings.min_keyframe_distance.to_value(),
             "next-file" => settings.next_file.to_value(),
+            "s3-uri" => {
+                let s3_uri = self.s3_uri.lock().unwrap();
+                let uri = match *s3_uri {
+                    Some(ref uri) => uri.to_string(),
+                    None => "".to_string(),
+                };
+
+                uri.to_value()
+            }
             _ => unimplemented!(),
         }
     }
@@ -860,13 +940,14 @@ impl BaseSinkImpl for S3PutObjectSink {
         let mut state = self.state.lock().unwrap();
         let settings = self.settings.lock().unwrap();
 
-        if let State::Started(ref mut started_state) = *state {
-            if settings.flush_on_error && started_state.need_flush {
-                drop(settings);
+        if let State::Started(ref mut started_state) = *state
+            && settings.flush_on_error
+            && started_state.need_flush
+        {
+            drop(settings);
 
-                if self.write_put_object_request(started_state).is_err() {
-                    gst::error!(CAT, imp = self, "Failed to finalize the next-file upload",);
-                }
+            if self.write_put_object_request(started_state).is_err() {
+                gst::error!(CAT, imp = self, "Failed to finalize the next-file upload",);
             }
         }
 
@@ -898,14 +979,14 @@ impl BaseSinkImpl for S3PutObjectSink {
                         Ok(_key_unit_event) => {
                             let mut state = self.state.lock().unwrap();
 
-                            if let State::Started(ref mut started_state) = *state {
-                                if let Err(e) = self.write_put_object_request(started_state) {
-                                    gst::element_imp_error!(
-                                        self,
-                                        gst::CoreError::Failed,
-                                        ["Failed to write on KeyUnitEvent, {e}"]
-                                    );
-                                }
+                            if let State::Started(ref mut started_state) = *state
+                                && let Err(e) = self.write_put_object_request(started_state)
+                            {
+                                gst::element_imp_error!(
+                                    self,
+                                    gst::CoreError::Failed,
+                                    ["Failed to write on KeyUnitEvent, {e}"]
+                                );
                             }
                         }
                         Err(e) => gst::error!(CAT, "Failed to parse key unit event: {e}"),
