@@ -13,51 +13,59 @@
 use super::CAT;
 use crate::s3utils::RUNTIME;
 use crate::transcriber::{
-    translate::{span_tokenize_items, TranslatedItem},
     TranslationTokenizationMethod,
+    translate::{TranslatedItem, span_tokenize_items},
 };
-use anyhow::{anyhow, Error};
+use anyhow::{Error, anyhow};
 use aws_sdk_s3::config::StalledStreamProtectionConfig;
 use aws_sdk_translate::error::ProvideErrorMetadata;
-use futures::future::{abortable, AbortHandle};
+use futures::future::{AbortHandle, abortable};
 use gst::subclass::prelude::*;
 use gst::{glib, prelude::*};
-use std::sync::mpsc::RecvTimeoutError;
 use std::sync::LazyLock;
-use std::sync::{mpsc, Mutex};
+use std::sync::{Mutex, MutexGuard};
 
 #[allow(deprecated)]
 static AWS_BEHAVIOR_VERSION: LazyLock<aws_config::BehaviorVersion> =
     LazyLock::new(aws_config::BehaviorVersion::v2023_11_09);
 
 const DEFAULT_LATENCY: gst::ClockTime = gst::ClockTime::from_seconds(2);
+const DEFAULT_LATENESS: gst::ClockTime = gst::ClockTime::from_seconds(0);
 const DEFAULT_REGION: &str = "us-east-1";
 const DEFAULT_INPUT_LANG_CODE: &str = "en-US";
 const DEFAULT_OUTPUT_LANG_CODE: &str = "fr-FR";
 const DEFAULT_TOKENIZATION_METHOD: TranslationTokenizationMethod =
     TranslationTokenizationMethod::SpanBased;
+const DEFAULT_BREVITY_ON: bool = false;
+
+const STANDARD_JOINABLE_PUNCTUATION: &str = "!.?,:;";
+const FRENCH_JOINABLE_PUNCTUATION: &str = ".,";
 
 #[derive(Debug, Clone)]
 struct Settings {
     latency: gst::ClockTime,
+    accumulator_lateness: gst::ClockTime,
     input_language_code: String,
     output_language_code: String,
     access_key: Option<String>,
     secret_access_key: Option<String>,
     session_token: Option<String>,
     tokenization_method: TranslationTokenizationMethod,
+    brevity_on: bool,
 }
 
 impl Default for Settings {
     fn default() -> Self {
         Self {
             latency: DEFAULT_LATENCY,
+            accumulator_lateness: DEFAULT_LATENESS,
             input_language_code: String::from(DEFAULT_INPUT_LANG_CODE),
             output_language_code: String::from(DEFAULT_OUTPUT_LANG_CODE),
             access_key: None,
             secret_access_key: None,
             session_token: None,
             tokenization_method: DEFAULT_TOKENIZATION_METHOD,
+            brevity_on: DEFAULT_BREVITY_ON,
         }
     }
 }
@@ -66,40 +74,24 @@ impl Default for Settings {
 struct InputItem {
     content: String,
     pts: gst::ClockTime,
-    rtime: gst::ClockTime,
     end_pts: gst::ClockTime,
-    is_punctuation: bool,
+    starts_with_joinable_punctuation: bool,
     discont: bool,
+    awstranscribe_item: Option<String>,
+    speechmatics_items: Option<Vec<String>>,
 }
 
 #[derive(Debug)]
 pub struct InputItems(Vec<InputItem>);
 
 impl InputItems {
-    fn start_rtime(&self) -> gst::ClockTime {
-        self.0.first().unwrap().rtime
-    }
-
-    fn start_pts(&self) -> gst::ClockTime {
-        self.0.first().unwrap().pts
+    fn start_pts(&self) -> Option<gst::ClockTime> {
+        self.0.first().map(|item| item.pts)
     }
 
     fn discont(&self) -> bool {
-        self.0.first().unwrap().discont
+        self.0.first().map(|item| item.discont).unwrap_or(false)
     }
-
-    fn end_pts(&self) -> gst::ClockTime {
-        self.0.last().unwrap().end_pts
-    }
-}
-
-enum TranslateInput {
-    Items(InputItems),
-    Gap {
-        pts: gst::ClockTime,
-        duration: Option<gst::ClockTime>,
-    },
-    Event(gst::Event),
 }
 
 enum TranslateOutput {
@@ -110,13 +102,11 @@ struct State {
     // (live, min, max)
     upstream_latency: Option<(bool, gst::ClockTime, Option<gst::ClockTime>)>,
     segment: Option<gst::FormattedSegment<gst::ClockTime>>,
-    accumulator: Option<InputItems>,
     client: Option<aws_sdk_translate::Client>,
     send_abort_handle: Option<AbortHandle>,
-    translate_tx: Option<mpsc::Sender<TranslateInput>>,
     discont: bool,
     seqnum: gst::Seqnum,
-    chained_one: bool,
+    current_speaker: Option<String>,
 }
 
 impl Default for State {
@@ -124,13 +114,11 @@ impl Default for State {
         Self {
             upstream_latency: None,
             segment: None,
-            accumulator: None,
             client: None,
             send_abort_handle: None,
-            translate_tx: None,
             discont: false,
             seqnum: gst::Seqnum::next(),
-            chained_one: false,
+            current_speaker: None,
         }
     }
 }
@@ -182,17 +170,10 @@ impl Translate {
             FlushStart(_) => {
                 gst::info!(CAT, imp = self, "received flush start, disconnecting");
                 let ret = gst::Pad::event_default(pad, Some(&*self.obj()), event);
-                let _ = self.state.lock().unwrap().translate_tx.take();
-                let _ = self.srcpad.pause_task();
                 self.disconnect();
                 ret
             }
             StreamStart(_) => {
-                if let Err(err) = self.start_srcpad_task() {
-                    gst::error!(CAT, imp = self, "Failed to start srcpad task: {err}");
-                    return false;
-                }
-
                 self.state.lock().unwrap().seqnum = event.seqnum();
 
                 gst::debug!(
@@ -204,19 +185,21 @@ impl Translate {
 
                 gst::Pad::event_default(pad, Some(&*self.obj()), event)
             }
+            Caps(_) => {
+                let caps = gst::Caps::builder("text/x-raw")
+                    .field("format", "utf8")
+                    .build();
+
+                let event = gst::event::Caps::builder(&caps)
+                    .seqnum(self.state.lock().unwrap().seqnum)
+                    .build();
+
+                self.srcpad.push_event(event)
+            }
             Segment(e) => {
                 {
                     let mut state = self.state.lock().unwrap();
 
-                    if state.segment.is_some() && state.chained_one {
-                        gst::element_imp_error!(
-                            self,
-                            gst::StreamError::Format,
-                            ["Multiple segments not supported"]
-                        );
-                        return false;
-                    }
-
                     let segment = match e.segment().clone().downcast::<gst::ClockTime>() {
                         Err(segment) => {
                             gst::element_imp_error!(
@@ -236,228 +219,46 @@ impl Translate {
 
                 gst::Pad::event_default(pad, Some(&*self.obj()), event)
             }
-            Caps(_) => {
-                let caps = gst::Caps::builder("text/x-raw")
-                    .field("format", "utf8")
-                    .build();
-
-                let event = gst::event::Caps::builder(&caps)
-                    .seqnum(self.state.lock().unwrap().seqnum)
-                    .build();
-
-                self.srcpad.push_event(event)
-            }
-            Gap(gap) => {
-                let state = self.state.lock().unwrap();
-
-                let (pts, duration) = gap.get();
-
-                if state.accumulator.is_none() {
-                    if let Some(translate_tx) = state.translate_tx.as_ref() {
-                        let _ = translate_tx.send(TranslateInput::Gap { pts, duration });
-                    }
-                }
-
-                true
-            }
-            Eos(_) => {
-                let translate_tx = self.state.lock().unwrap().translate_tx.take();
-                if let Some(translate_tx) = translate_tx {
-                    gst::debug!(CAT, imp = self, "received EOS, draining");
-
-                    let _ = translate_tx.send(TranslateInput::Items(self.drain(false)));
-                }
-
-                true
-            }
             CustomDownstream(c) => {
                 let Some(s) = c.structure() else {
                     return gst::Pad::event_default(pad, Some(&*self.obj()), event);
                 };
 
-                let drain = match s.name().as_str() {
-                    "rstranscribe/final-transcript" => {
-                        gst::debug!(CAT, imp = self, "transcript is final, draining");
-                        true
-                    }
-                    "rstranscribe/speaker-change" => {
-                        gst::debug!(CAT, imp = self, "speaker change, draining");
-                        true
-                    }
-                    _ => false,
-                };
-
-                if drain {
-                    let items = self.drain(false);
-
-                    if let Some(translate_tx) = self.state.lock().unwrap().translate_tx.as_ref() {
-                        let _ = translate_tx.send(TranslateInput::Items(items));
-                        let _ = translate_tx.send(TranslateInput::Event(event));
-                    }
-
-                    true
-                } else {
-                    gst::Pad::event_default(pad, Some(&*self.obj()), event)
+                if s.name().as_str() == "rstranscribe/speaker-change" {
+                    gst::debug!(CAT, imp = self, "speaker change, draining");
+                    self.state.lock().unwrap().current_speaker = s.get::<String>("speaker").ok();
                 }
+
+                gst::Pad::event_default(pad, Some(&*self.obj()), event)
             }
             _ => gst::Pad::event_default(pad, Some(&*self.obj()), event),
         }
     }
 
-    fn maybe_send(&self) -> Result<gst::FlowSuccess, gst::FlowError> {
-        let Some(now) = self.obj().current_running_time() else {
-            gst::trace!(
-                CAT,
-                imp = self,
-                "no current running time, we cannot be late"
-            );
-            return Ok(gst::FlowSuccess::Ok);
-        };
-
-        let Some(upstream_latency) = self.upstream_latency() else {
-            gst::trace!(CAT, imp = self, "no upstream latency, we cannot be late");
-            return Ok(gst::FlowSuccess::Ok);
-        };
-
-        let (upstream_live, upstream_min, _) = upstream_latency;
-
-        if !upstream_live {
-            gst::trace!(CAT, imp = self, "upstream isn't live, we are not late");
-            return Ok(gst::FlowSuccess::Ok);
-        }
-
-        loop {
-            let start_rtime = self
-                .state
-                .lock()
-                .unwrap()
-                .accumulator
-                .as_ref()
-                .map(|accumulator| accumulator.start_rtime());
-
-            if let Some(start_rtime) = start_rtime {
-                if start_rtime + upstream_min < now {
-                    gst::debug!(
-                        CAT,
-                        imp = self,
-                        "draining on timeout: {start_rtime} + {upstream_min} < {now}",
-                    );
-                    self.do_send(self.drain(true))?;
-                } else {
-                    gst::trace!(
-                        CAT,
-                        imp = self,
-                        "queued content is not late: {start_rtime} + {upstream_min} >= {now}"
-                    );
-                    return Ok(gst::FlowSuccess::Ok);
-                }
-            } else {
-                gst::trace!(CAT, imp = self, "no queued content, cannot be late");
-                return Ok(gst::FlowSuccess::Ok);
-            }
-        }
-    }
-
-    fn do_send(&self, to_translate: InputItems) -> Result<gst::FlowSuccess, gst::FlowError> {
-        if to_translate.0.is_empty() {
-            gst::trace!(CAT, imp = self, "nothing to send, returning early");
-            return Ok(gst::FlowSuccess::Ok);
-        }
-
-        let (future, abort_handle) = abortable(self.send(to_translate));
-
-        self.state.lock().unwrap().send_abort_handle = Some(abort_handle);
-
-        match RUNTIME.block_on(future) {
-            Err(_) => {
-                gst::debug!(CAT, imp = self, "send aborted, returning flushing");
-                Err(gst::FlowError::Flushing)
-            }
-            Ok(res) => match res {
-                Err(e) => {
-                    gst::element_imp_error!(
-                        self,
-                        gst::StreamError::Failed,
-                        ["Failed sending data: {}", e]
-                    );
-                    Err(gst::FlowError::Error)
-                }
-                Ok(mut output) => {
-                    for item in output.drain(..) {
-                        match item {
-                            TranslateOutput::Item(buffer) => {
-                                gst::debug!(CAT, imp = self, "pushing {buffer:?}");
-                                self.srcpad.push(buffer)?;
-                            }
-                        }
-                    }
-
-                    Ok(gst::FlowSuccess::Ok)
-                }
-            },
-        }
-    }
-
-    fn drain(&self, up_to_punctuation: bool) -> InputItems {
-        let mut state = self.state.lock().unwrap();
-
-        let Some(accumulator) = state.accumulator.take() else {
-            gst::trace!(CAT, imp = self, "accumulator is empty");
-            return InputItems(vec![]);
-        };
-
-        let items = match up_to_punctuation {
-            true => {
-                if let Some(punctuation_index) =
-                    accumulator.0.iter().rposition(|item| item.is_punctuation)
-                {
-                    let (items, trailing) = accumulator.0.split_at(punctuation_index + 1);
-
-                    if !trailing.is_empty() {
-                        state.accumulator = Some(InputItems(trailing.to_vec()));
-                    }
-
-                    gst::log!(CAT, imp = self, "drained up to punctuation: {items:?}");
-
-                    InputItems(items.to_vec())
-                } else {
-                    gst::log!(CAT, imp = self, "drained all items: {accumulator:?}");
-                    accumulator
-                }
-            }
-            false => {
-                gst::log!(CAT, imp = self, "drained all items: {accumulator:?}");
-
-                accumulator
-            }
-        };
-
-        items
-    }
-
     async fn send(&self, to_translate: InputItems) -> Result<Vec<TranslateOutput>, Error> {
-        let (input_lang, output_lang, latency, tokenization_method) = {
+        let (input_lang, output_lang, latency, tokenization_method, lateness, brevity_on) = {
             let settings = self.settings.lock().unwrap();
             (
                 settings.input_language_code.clone(),
                 settings.output_language_code.clone(),
                 settings.latency,
                 settings.tokenization_method,
+                settings.accumulator_lateness,
+                settings.brevity_on,
             )
         };
 
-        let (client, segment) = {
-            let mut state = self.state.lock().unwrap();
+        let (client, segment, speaker) = {
+            let state = self.state.lock().unwrap();
 
-            state
-                .segment
-                .as_mut()
-                .unwrap()
-                .set_position(Some(to_translate.end_pts()));
+            let Some(segment) = state.segment.clone() else {
+                return Err(anyhow!("buffer received before segment"));
+            };
 
             (
                 state.client.as_ref().unwrap().clone(),
-                state.segment.as_ref().unwrap().clone(),
+                segment,
+                state.current_speaker.clone(),
             )
         };
 
@@ -479,10 +280,15 @@ impl Translate {
         let mut content: Vec<String> = vec![];
         let mut it = to_translate.0.iter().peekable();
 
+        let items: [String; 0] = [];
+        let mut original_aws_items = gst::List::new(items);
+        let items: [String; 0] = [];
+        let mut original_speechmatics_items = gst::List::new(items);
+
         while let Some(item) = it.next() {
             let suffix = match it.peek() {
                 Some(next_item) => {
-                    if next_item.is_punctuation {
+                    if next_item.starts_with_joinable_punctuation {
                         ""
                     } else {
                         " "
@@ -497,20 +303,39 @@ impl Translate {
                     format!("{SPAN_START}{}{SPAN_END}{}", item.content, suffix)
                 }
             });
+
+            if let Some(ref original_item) = item.awstranscribe_item {
+                original_aws_items.append(original_item.to_send_value());
+            }
+
+            if let Some(ref original_items) = item.speechmatics_items {
+                for original_item in original_items {
+                    original_speechmatics_items.append(original_item.to_send_value());
+                }
+            }
         }
 
         let content: String = content.join("");
 
-        gst::log!(
+        gst::debug!(
             CAT,
             imp = self,
             "translating {content} with duration list: {ts_duration_list:?}"
         );
 
+        let mut translate_settings_builder =
+            aws_sdk_translate::types::TranslationSettings::builder();
+
+        if brevity_on {
+            translate_settings_builder =
+                translate_settings_builder.set_brevity(Some(aws_sdk_translate::types::Brevity::On));
+        }
+
         let translated_text = client
             .translate_text()
             .set_source_language_code(Some(input_lang))
             .set_target_language_code(Some(output_lang.clone()))
+            .set_settings(Some(translate_settings_builder.build()))
             .set_text(Some(content))
             .send()
             .await
@@ -519,17 +344,6 @@ impl Translate {
 
         gst::log!(CAT, imp = self, "translation received: {translated_text}");
 
-        let s = gst::Structure::builder("awstranslate/raw")
-            .field("translation", &translated_text)
-            .field("arrival-time", self.obj().current_running_time())
-            .field("start-time", to_translate.start_pts())
-            .field("language-code", &output_lang)
-            .build();
-
-        let _ = self
-            .obj()
-            .post_message(gst::message::Element::builder(s).src(&*self.obj()).build());
-
         let upstream_latency = self.upstream_latency();
 
         let mut translated_items = match tokenization_method {
@@ -561,23 +375,31 @@ impl Translate {
 
         gst::log!(CAT, imp = self, "translated items: {translated_items:?}");
 
-        for mut item in translated_items.drain(..) {
-            if let Some((upstream_live, upstream_min, _)) = upstream_latency {
-                if upstream_live {
-                    if let Some(now) = self.obj().current_running_time() {
-                        let start_rtime = segment.to_running_time(item.pts).unwrap();
-                        let deadline = start_rtime + upstream_min + latency;
+        let mut translated_items_builder = gst::Structure::builder("awstranslate/items");
 
-                        if deadline < now {
-                            let adjusted_pts = item.pts + now - deadline;
-                            gst::warning!(
-                                CAT,
-                                imp = self,
-                                "text translated too late, adjusting timestamp {} -> {adjusted_pts}", item.pts,
-                            );
-                            item.pts = adjusted_pts;
-                        }
-                    }
+        let mut end_pts: Option<gst::ClockTime> = None;
+
+        for mut item in translated_items.drain(..) {
+            translated_items_builder = translated_items_builder.field(&item.content, item.pts);
+
+            item.pts += lateness;
+
+            if let Some((upstream_live, upstream_min, _)) = upstream_latency
+                && upstream_live
+                && let Some(now) = self.obj().current_running_time()
+            {
+                let start_rtime = segment.to_running_time(item.pts).unwrap();
+                let deadline = start_rtime + upstream_min + latency;
+
+                if deadline < now {
+                    let adjusted_pts = item.pts + now - deadline;
+                    gst::warning!(
+                        CAT,
+                        imp = self,
+                        "text translated too late, adjusting timestamp {} -> {adjusted_pts}",
+                        item.pts,
+                    );
+                    item.pts = adjusted_pts;
                 }
             }
 
@@ -593,9 +415,47 @@ impl Translate {
                 }
             }
 
+            end_pts = Some(item.pts);
+
             output.push(TranslateOutput::Item(buf));
         }
 
+        let mut message_builder = gst::Structure::builder("awstranslate/raw")
+            .field("translation", translated_items_builder.build())
+            .field("arrival-time", self.obj().current_running_time())
+            .field("start-time", to_translate.start_pts())
+            .field("language-code", &output_lang);
+
+        if let Some(speaker) = speaker {
+            message_builder = message_builder.field("speaker", speaker);
+        }
+
+        if !original_aws_items.is_empty() {
+            message_builder =
+                message_builder.field("original-awstranscribe-items", original_aws_items);
+        }
+
+        if !original_speechmatics_items.is_empty() {
+            message_builder =
+                message_builder.field("original-speechmatics-items", original_speechmatics_items);
+        }
+
+        let _ = self.obj().post_message(
+            gst::message::Element::builder(message_builder.build())
+                .src(&*self.obj())
+                .build(),
+        );
+
+        if let Some(end_pts) = end_pts {
+            self.state
+                .lock()
+                .unwrap()
+                .segment
+                .as_mut()
+                .unwrap()
+                .set_position(end_pts);
+        }
+
         Ok(output)
     }
 
@@ -609,100 +469,6 @@ impl Translate {
         Ok(())
     }
 
-    fn start_srcpad_task(&self) -> Result<(), gst::LoggableError> {
-        gst::debug!(CAT, imp = self, "starting source pad task");
-
-        self.ensure_connection()
-            .map_err(|err| gst::loggable_error!(CAT, "Failed to start pad task: {err}"))?;
-
-        let (translate_tx, translate_rx) = mpsc::channel();
-
-        self.state.lock().unwrap().translate_tx = Some(translate_tx);
-
-        let this_weak = self.downgrade();
-        let res = self.srcpad.start_task(move || loop {
-            let Some(this) = this_weak.upgrade() else {
-                break;
-            };
-
-            let timeout = match this.upstream_latency() {
-                Some((true, _min, _max)) => std::time::Duration::from_millis(100),
-                _ => std::time::Duration::MAX,
-            };
-
-            gst::trace!(CAT, imp = this, "now waiting with timeout {timeout:?}");
-
-            match translate_rx.recv_timeout(timeout) {
-                Ok(input) => {
-                    gst::trace!(CAT, imp = this, "received input on translate queue");
-
-                    match input {
-                        TranslateInput::Items(to_translate) => {
-                            if let Err(err) = this.do_send(to_translate) {
-                                if err != gst::FlowError::Flushing {
-                                    gst::element_error!(
-                                        this.obj(),
-                                        gst::StreamError::Failed,
-                                        ["Streaming failed: {}", err]
-                                    );
-                                }
-                                let _ = this.srcpad.pause_task();
-                            }
-                        }
-                        TranslateInput::Gap { pts, duration } => {
-                            let event = gst::event::Gap::builder(pts)
-                                .duration(duration)
-                                .seqnum(this.state.lock().unwrap().seqnum)
-                                .build();
-
-                            let _ = this.srcpad.push_event(event);
-                        }
-                        TranslateInput::Event(event) => {
-                            gst::debug!(CAT, imp = this, "Forwarding event {event:?}");
-                            let _ = this.srcpad.push_event(event);
-                        }
-                    }
-                }
-                Err(RecvTimeoutError::Timeout) => {
-                    gst::trace!(
-                        CAT,
-                        imp = this,
-                        "timed out waiting for input on translate queue"
-                    );
-
-                    if let Err(err) = this.maybe_send() {
-                        if err != gst::FlowError::Flushing {
-                            gst::element_error!(
-                                this.obj(),
-                                gst::StreamError::Failed,
-                                ["Streaming failed: {}", err]
-                            );
-                        }
-                        let _ = this.srcpad.pause_task();
-                    }
-                }
-                Err(RecvTimeoutError::Disconnected) => {
-                    gst::log!(CAT, imp = this, "translate queue disconnected, pushing EOS");
-
-                    let event = gst::event::Eos::builder()
-                        .seqnum(this.state.lock().unwrap().seqnum)
-                        .build();
-                    let _ = this.srcpad.push_event(event);
-                    let _ = this.srcpad.pause_task();
-                    break;
-                }
-            }
-        });
-
-        if res.is_err() {
-            return Err(gst::loggable_error!(CAT, "Failed to start pad task"));
-        }
-
-        gst::debug!(CAT, imp = self, "started source pad task");
-
-        Ok(())
-    }
-
     fn disconnect(&self) {
         let mut state = self.state.lock().unwrap();
 
@@ -711,104 +477,184 @@ impl Translate {
             abort_handle.abort();
         }
 
-        *state = State::default();
+        *state = State::default()
     }
 
-    fn sink_chain(
+    fn input_item_from_buffer(
         &self,
-        pad: &gst::Pad,
-        buffer: gst::Buffer,
-    ) -> Result<gst::FlowSuccess, gst::FlowError> {
+        state: &mut State,
+        buffer: &gst::BufferRef,
+        is_french: bool,
+    ) -> Result<Option<InputItem>, gst::FlowError> {
+        let Some(pts) = buffer.pts() else {
+            gst::warning!(CAT, imp = self, "dropping first buffer without a PTS");
+            return Ok(None);
+        };
+
+        let end_pts = match buffer.duration() {
+            Some(duration) => duration + pts,
+            _ => pts,
+        };
+
         let data = buffer.map_readable().map_err(|_| {
-            gst::error!(CAT, obj = pad, "Can't map buffer readable");
+            gst::error!(CAT, imp = self, "Can't map buffer readable");
 
             gst::FlowError::Error
         })?;
 
         let data = String::from_utf8(data.to_vec()).map_err(|err| {
-            gst::error!(CAT, obj = pad, "Can't decode utf8: {}", err);
+            gst::error!(CAT, imp = self, "Can't decode utf8: {}", err);
 
             gst::FlowError::Error
         })?;
 
-        let drained_items = if buffer.flags().contains(gst::BufferFlags::DISCONT) {
-            let items = self.drain(false);
+        let original_aws_item: Option<String> =
+            gst::meta::CustomMeta::from_buffer(buffer, "AWSTranscribeItemMeta")
+                .ok()
+                .and_then(|m| m.structure().get::<String>("item").ok());
 
-            gst::log!(CAT, imp = self, "draining on discont");
+        let original_speechmatics_items: Option<Vec<String>> =
+            gst::meta::CustomMeta::from_buffer(buffer, "SpeechmaticsItemMeta")
+                .ok()
+                .and_then(|m| m.structure().get::<Vec<String>>("items").ok());
 
-            Some(items)
+        let joinable_punctuation = if is_french {
+            FRENCH_JOINABLE_PUNCTUATION
         } else {
-            None
+            STANDARD_JOINABLE_PUNCTUATION
         };
 
-        {
-            let mut state = self.state.lock().unwrap();
+        let starts_with_joinable_punctuation = data
+            .chars()
+            .next()
+            .map(|c| joinable_punctuation.contains(c))
+            .unwrap_or(false);
 
-            if let Some(items) = drained_items {
-                if let Some(translate_tx) = state.translate_tx.as_ref() {
-                    let _ = translate_tx.send(TranslateInput::Items(items));
+        let discont = state.discont;
+        state.discont = false;
+
+        Ok(Some(InputItem {
+            content: data,
+            pts,
+            end_pts,
+            starts_with_joinable_punctuation,
+            discont,
+            awstranscribe_item: original_aws_item,
+            speechmatics_items: original_speechmatics_items,
+        }))
+    }
+
+    fn do_send(
+        &self,
+        mut state: MutexGuard<State>,
+        to_translate: InputItems,
+    ) -> Result<gst::FlowSuccess, gst::FlowError> {
+        let (future, abort_handle) = abortable(self.send(to_translate));
+
+        state.send_abort_handle = Some(abort_handle);
+
+        drop(state);
+
+        match RUNTIME.block_on(future) {
+            Err(_) => {
+                gst::debug!(CAT, imp = self, "send aborted, returning flushing");
+                Err(gst::FlowError::Flushing)
+            }
+            Ok(res) => match res {
+                Err(e) => {
+                    gst::element_imp_error!(
+                        self,
+                        gst::StreamError::Failed,
+                        ["Failed sending data: {}", e]
+                    );
+                    Err(gst::FlowError::Error)
                 }
+                Ok(mut output) => {
+                    let mut bufferlist = gst::BufferList::new();
+                    let bufferlist_mut = bufferlist.get_mut().unwrap();
+                    for item in output.drain(..) {
+                        match item {
+                            TranslateOutput::Item(buffer) => {
+                                gst::debug!(CAT, imp = self, "pushing {buffer:?}");
+                                bufferlist_mut.add(buffer);
+                            }
+                        }
+                    }
 
-                state.discont = true;
-            }
+                    self.srcpad.push_list(bufferlist)
+                }
+            },
+        }
+    }
 
-            let Some(segment) = state.segment.as_ref() else {
-                gst::warning!(CAT, imp = self, "dropping buffer before segment");
-                return Ok(gst::FlowSuccess::Ok);
-            };
-
-            let Some(pts) = buffer.pts() else {
-                gst::warning!(CAT, imp = self, "dropping first buffer without a PTS");
-                return Ok(gst::FlowSuccess::Ok);
-            };
-
-            let end_pts = match buffer.duration() {
-                Some(duration) => duration + pts,
-                _ => pts,
-            };
-
-            let Some(rtime) = segment.to_running_time(pts) else {
-                gst::log!(CAT, imp = self, "clipping buffer outside segment");
-                return Ok(gst::FlowSuccess::Ok);
-            };
-
-            let is_punctuation = data.chars().all(|c| c.is_ascii_punctuation());
-
-            let discont = state.discont;
-            state.discont = false;
-
-            let item = InputItem {
-                content: data,
-                pts,
-                rtime,
-                end_pts,
-                is_punctuation,
-                discont,
-            };
-
-            if let Some(accumulator) = state.accumulator.as_mut() {
-                gst::log!(CAT, imp = self, "queuing item on accumulator: {item:?}");
-                accumulator.0.push(item);
-            } else {
-                gst::log!(
-                    CAT,
-                    imp = self,
-                    "creating new accumulator with item: {item:?}"
-                );
-                state.accumulator = Some(InputItems(vec![item]));
-            }
-
-            state.chained_one = true;
-
-            gst::trace!(
-                CAT,
-                imp = self,
-                "accumulator is now {:#?}",
-                state.accumulator
+    fn sink_chain(
+        &self,
+        _pad: &gst::Pad,
+        buffer: gst::Buffer,
+    ) -> Result<gst::FlowSuccess, gst::FlowError> {
+        self.ensure_connection().map_err(|err| {
+            gst::error!(CAT, "Failed to connect to AWS: {err:?}");
+            gst::element_imp_error!(
+                self,
+                gst::StreamError::Failed,
+                ["Streaming failed: {}", err]
             );
 
-            Ok(gst::FlowSuccess::Ok)
+            gst::FlowError::Error
+        })?;
+
+        let is_french = self
+            .settings
+            .lock()
+            .unwrap()
+            .input_language_code
+            .starts_with("fr-");
+
+        let mut state = self.state.lock().unwrap();
+
+        let to_translate =
+            if let Some(item) = self.input_item_from_buffer(&mut state, &buffer, is_french)? {
+                vec![item]
+            } else {
+                return Ok(gst::FlowSuccess::Ok);
+            };
+
+        self.do_send(state, InputItems(to_translate))
+    }
+
+    fn sink_chain_list(
+        &self,
+        _pad: &gst::Pad,
+        bufferlist: gst::BufferList,
+    ) -> Result<gst::FlowSuccess, gst::FlowError> {
+        self.ensure_connection().map_err(|err| {
+            gst::error!(CAT, "Failed to connect to AWS: {err:?}");
+            gst::element_imp_error!(
+                self,
+                gst::StreamError::Failed,
+                ["Streaming failed: {}", err]
+            );
+
+            gst::FlowError::Error
+        })?;
+
+        let is_french = self
+            .settings
+            .lock()
+            .unwrap()
+            .input_language_code
+            .starts_with("fr-");
+
+        let mut state = self.state.lock().unwrap();
+
+        let mut to_translate: Vec<InputItem> = vec![];
+        for buffer in bufferlist.iter() {
+            if let Some(item) = self.input_item_from_buffer(&mut state, buffer, is_french)? {
+                to_translate.push(item);
+            }
         }
+
+        self.do_send(state, InputItems(to_translate))
     }
 
     fn prepare(&self) -> Result<(), gst::ErrorMessage> {
@@ -905,6 +751,13 @@ impl ObjectSubclass for Translate {
                     |translate| translate.sink_chain(pad, buffer),
                 )
             })
+            .chain_list_function(|pad, parent, buffer| {
+                Translate::catch_panic_pad_function(
+                    parent,
+                    || Err(gst::FlowError::Error),
+                    |translate| translate.sink_chain_list(pad, buffer),
+                )
+            })
             .event_function(|pad, parent, event| {
                 Translate::catch_panic_pad_function(
                     parent,
@@ -942,11 +795,27 @@ impl ObjectImpl for Translate {
             vec![
                 glib::ParamSpecUInt::builder("latency")
                     .nick("Latency")
-                    .blurb("Amount of milliseconds to allow AWS Polly")
+                    .blurb("Amount of milliseconds to allow AWS translate")
                     .default_value(DEFAULT_LATENCY.mseconds() as u32)
                     .mutable_ready()
                     .deprecated()
                     .build(),
+                /**
+                 * gstawstranslate:accumulator-lateness
+                 *
+                 * The timestamps of the translated text will be shifted forward
+                 * by the value of this property.
+                 *
+                 * Deprecated:plugins-rs-0.15.0: use a textaccumulate element upstream instead
+                 * Since: plugins-rs-0.14.0
+                 */
+                glib::ParamSpecUInt::builder("accumulator-lateness")
+                    .nick("Lateness")
+                    .blurb("By how much to shift input timestamps forward")
+                    .default_value(DEFAULT_LATENESS.mseconds() as u32)
+                    .mutable_ready()
+                    .deprecated()
+                    .build(),
                 glib::ParamSpecString::builder("access-key")
                     .nick("Access Key")
                     .blurb("AWS Access Key")
@@ -980,6 +849,21 @@ impl ObjectImpl for Translate {
                     .default_value(DEFAULT_TOKENIZATION_METHOD)
                     .mutable_ready()
                     .build(),
+                /**
+                 * gstawstranslate:brevity-on
+                 *
+                 * Turn on the brevity feature, only available for some languages.
+                 *
+                 * https://docs.aws.amazon.com/translate/latest/dg/customizing-translations-brevity.html
+                 *
+                 * since: plugins-rs-0.15.0
+                 */
+                glib::ParamSpecBoolean::builder("brevity-on")
+                    .nick("Brevity On")
+                    .blurb("Whether brevity should be turned on")
+                    .default_value(DEFAULT_BREVITY_ON)
+                    .mutable_ready()
+                    .build(),
             ]
         });
 
@@ -1002,6 +886,12 @@ impl ObjectImpl for Translate {
                     value.get::<u32>().expect("type checked upstream").into(),
                 );
             }
+            "accumulator-lateness" => {
+                let mut settings = self.settings.lock().unwrap();
+                settings.accumulator_lateness = gst::ClockTime::from_mseconds(
+                    value.get::<u32>().expect("type checked upstream").into(),
+                );
+            }
             "access-key" => {
                 let mut settings = self.settings.lock().unwrap();
                 settings.access_key = value.get().expect("type checked upstream");
@@ -1027,6 +917,7 @@ impl ObjectImpl for Translate {
             "tokenization-method" => {
                 self.settings.lock().unwrap().tokenization_method = value.get().unwrap()
             }
+            "brevity-on" => self.settings.lock().unwrap().brevity_on = value.get().unwrap(),
             _ => unimplemented!(),
         }
     }
@@ -1037,6 +928,10 @@ impl ObjectImpl for Translate {
                 let settings = self.settings.lock().unwrap();
                 (settings.latency.mseconds() as u32).to_value()
             }
+            "accumulator-lateness" => {
+                let settings = self.settings.lock().unwrap();
+                (settings.accumulator_lateness.mseconds() as u32).to_value()
+            }
             "access-key" => {
                 let settings = self.settings.lock().unwrap();
                 settings.access_key.to_value()
@@ -1058,6 +953,7 @@ impl ObjectImpl for Translate {
                 settings.output_language_code.to_value()
             }
             "tokenization-method" => self.settings.lock().unwrap().tokenization_method.to_value(),
+            "brevity-on" => self.settings.lock().unwrap().brevity_on.to_value(),
             _ => unimplemented!(),
         }
     }
@@ -1124,7 +1020,6 @@ impl ElementImpl for Translate {
             }
             gst::StateChange::PausedToReady => {
                 self.disconnect();
-                let _ = self.srcpad.stop_task();
             }
             _ => (),
         }
