@@ -198,7 +198,7 @@ impl OnvifMetadataParse {
         if state
             .in_segment
             .position()
-            .map_or(true, |position| position < pts)
+            .is_none_or(|position| position < pts)
         {
             gst::trace!(CAT, imp = self, "Input position updated to {}", pts);
             state.in_segment.set_position(pts);
@@ -290,16 +290,15 @@ impl OnvifMetadataParse {
                             None
                         }
                     })
+                    && running_time.saturating_sub(front_running_time) >= state.configured_latency
                 {
-                    if running_time.saturating_sub(front_running_time) >= state.configured_latency {
-                        gst::warning!(
-                            CAT,
-                            obj = pad,
-                            "Received no UTC time in the first {}",
-                            state.configured_latency
-                        );
-                        state.pre_queued_buffers.remove(idx);
-                    }
+                    gst::warning!(
+                        CAT,
+                        obj = pad,
+                        "Received no UTC time in the first {}",
+                        state.configured_latency
+                    );
+                    state.pre_queued_buffers.remove(idx);
                 }
 
                 return Ok(gst::FlowSuccess::Ok);
@@ -321,7 +320,7 @@ impl OnvifMetadataParse {
         buffer: gst::Buffer,
         running_time: gst::Signed<gst::ClockTime>,
     ) -> Result<(), gst::FlowError> {
-        let State {
+        let &mut State {
             ref mut pre_queued_buffers,
             ref mut queued_frames,
             ref utc_time_running_time_mapping,
@@ -571,7 +570,7 @@ impl OnvifMetadataParse {
             if state
                 .clock_wait
                 .as_ref()
-                .map_or(true, |clock_wait| clock_wait.time() != earliest_clock_time)
+                .is_none_or(|clock_wait| clock_wait.time() != earliest_clock_time)
             {
                 if let Some(clock_wait) = state.clock_wait.take() {
                     clock_wait.unschedule();
@@ -603,8 +602,8 @@ impl OnvifMetadataParse {
         drain_utc_time: Option<gst::ClockTime>,
     ) -> Result<Vec<BufferOrEvent>, gst::FlowError> {
         let State {
-            ref mut queued_frames,
-            ref mut out_segment,
+            queued_frames,
+            out_segment,
             utc_time_running_time_mapping,
             ..
         } = &mut *state;
@@ -682,7 +681,7 @@ impl OnvifMetadataParse {
 
                         if out_segment
                             .position()
-                            .map_or(true, |position| position < current_position)
+                            .is_none_or(|position| position < current_position)
                         {
                             gst::trace!(
                                 CAT,
@@ -768,7 +767,7 @@ impl OnvifMetadataParse {
 
             if out_segment
                 .position()
-                .map_or(true, |position| position < frame_pts)
+                .is_none_or(|position| position < frame_pts)
             {
                 gst::trace!(CAT, imp = self, "Output position updated to {}", frame_pts);
                 out_segment.set_position(frame_pts);
@@ -959,7 +958,7 @@ impl OnvifMetadataParse {
                         if state
                             .in_segment
                             .position()
-                            .map_or(true, |position| position < current_position)
+                            .is_none_or(|position| position < current_position)
                         {
                             gst::trace!(
                                 CAT,
@@ -973,8 +972,8 @@ impl OnvifMetadataParse {
                     _ => (),
                 }
 
-                let State {
-                    utc_time_running_time_mapping,
+                let &mut State {
+                    ref mut utc_time_running_time_mapping,
                     ref in_segment,
                     ref mut queued_frames,
                     ref mut pre_queued_buffers,
@@ -1280,7 +1279,7 @@ impl OnvifMetadataParse {
                 // Drain until the current clock running time minus the configured latency when
                 // live
                 if let Some((now, base_time)) = Option::zip(
-                    self.obj().clock().and_then(|clock| clock.time()),
+                    self.obj().clock().as_ref().map(gst::Clock::time),
                     self.obj().base_time(),
                 ) {
                     gst::trace!(
@@ -1342,45 +1341,55 @@ impl OnvifMetadataParse {
                     gst::debug!(CAT, imp = self, "EOS, waiting on cond");
                     state = self.cond.wait(state).unwrap();
                     gst::trace!(CAT, imp = self, "Woke up");
-                } else if let Some(clock_wait) = state.clock_wait.clone() {
-                    gst::trace!(
-                        CAT,
-                        imp = self,
-                        "Waiting on timer with time {}, now {}",
-                        clock_wait.time(),
-                        clock_wait.clock().and_then(|clock| clock.time()).display(),
-                    );
-
-                    drop(state);
-                    let res = clock_wait.wait();
-                    state = self.state.lock().unwrap();
-
-                    // Unset clock wait if it didn't change in the meantime: waiting again
-                    // on it is going to return immediately and instead if there's nothing
-                    // new to drain then we need to wait on the condvar
-                    if state.clock_wait.as_ref() == Some(&clock_wait) {
-                        state.clock_wait = None;
-                    }
-
-                    match res {
-                        (Ok(_), jitter) => {
-                            gst::trace!(CAT, imp = self, "Woke up after waiting for {}", jitter);
-                            last_clock_wait_time = Some(clock_wait.time());
-                        }
-                        (Err(err), jitter) => {
+                } else {
+                    match state.clock_wait.clone() {
+                        Some(clock_wait) => {
                             gst::trace!(
                                 CAT,
                                 imp = self,
-                                "Woke up with error {:?} and jitter {}",
-                                err,
-                                jitter
+                                "Waiting on timer with time {}, now {}",
+                                clock_wait.time(),
+                                clock_wait.clock().as_ref().map(gst::Clock::time).display(),
                             );
+
+                            drop(state);
+                            let res = clock_wait.wait();
+                            state = self.state.lock().unwrap();
+
+                            // Unset clock wait if it didn't change in the meantime: waiting again
+                            // on it is going to return immediately and instead if there's nothing
+                            // new to drain then we need to wait on the condvar
+                            if state.clock_wait.as_ref() == Some(&clock_wait) {
+                                state.clock_wait = None;
+                            }
+
+                            match res {
+                                (Ok(_), jitter) => {
+                                    gst::trace!(
+                                        CAT,
+                                        imp = self,
+                                        "Woke up after waiting for {}",
+                                        jitter
+                                    );
+                                    last_clock_wait_time = Some(clock_wait.time());
+                                }
+                                (Err(err), jitter) => {
+                                    gst::trace!(
+                                        CAT,
+                                        imp = self,
+                                        "Woke up with error {:?} and jitter {}",
+                                        err,
+                                        jitter
+                                    );
+                                }
+                            }
+                        }
+                        _ => {
+                            gst::debug!(CAT, imp = self, "Waiting on cond");
+                            state = self.cond.wait(state).unwrap();
+                            gst::trace!(CAT, imp = self, "Woke up");
                         }
                     }
-                } else {
-                    gst::debug!(CAT, imp = self, "Waiting on cond");
-                    state = self.cond.wait(state).unwrap();
-                    gst::trace!(CAT, imp = self, "Woke up");
                 }
 
                 // And retry if there's anything to drain now.
