@@ -7,24 +7,26 @@
 //
 // SPDX-License-Identifier: MPL-2.0
 
+use crate::quinnconnection::*;
 use crate::quinnquicmeta::QuinnQuicMeta;
 use crate::quinnquicquery::*;
 use crate::utils::{
-    client_endpoint, get_stats, make_socket_addr, server_endpoint, wait, Canceller,
-    QuinnQuicEndpointConfig, WaitError, CONNECTION_CLOSE_CODE, CONNECTION_CLOSE_MSG, RUNTIME,
+    CONNECTION_CLOSE_CODE, CONNECTION_CLOSE_MSG, Canceller, RUNTIME, WaitError, get_stats,
+    make_socket_addr, wait,
 };
 use crate::{common::*, utils};
-use async_channel::{unbounded, Receiver, Sender};
+use async_channel::{Receiver, Sender, unbounded};
 use bytes::Bytes;
-use futures::{future::BoxFuture, stream::FuturesUnordered, StreamExt};
+use futures::{StreamExt, future::BoxFuture, stream::FuturesUnordered};
 use gst::{glib, prelude::*, subclass::prelude::*};
 use gst_base::prelude::*;
 use gst_base::subclass::base_src::CreateSuccess;
 use gst_base::subclass::prelude::*;
-use quinn::{Chunk, Connection, ConnectionError, ReadError, RecvStream, TransportConfig, VarInt};
+use quinn::{Connection, ConnectionError, ReadError, RecvStream, VarInt};
+use std::net::SocketAddr;
 use std::path::PathBuf;
-use std::sync::{LazyLock, Mutex};
-use std::thread::{spawn, Builder, JoinHandle};
+use std::sync::{Arc, LazyLock, Mutex};
+use std::thread::{Builder, JoinHandle};
 use tokio::sync::oneshot;
 
 const DEFAULT_ROLE: QuinnQuicRole = QuinnQuicRole::Server;
@@ -56,6 +58,7 @@ struct Started {
     // implement clone.
     data_rx: Option<Receiver<QuinnData>>,
     thread_quit: Option<oneshot::Sender<()>>,
+    socket_addr: SocketAddr,
 }
 
 #[derive(Default)]
@@ -81,6 +84,7 @@ struct Settings {
     use_datagram: bool,
     certificate_file: Option<PathBuf>,
     private_key_file: Option<PathBuf>,
+    certificate_database_file: Option<PathBuf>,
     transport_config: QuinnQuicTransportConfig,
 }
 
@@ -101,6 +105,7 @@ impl Default for Settings {
             use_datagram: DEFAULT_USE_DATAGRAM,
             certificate_file: None,
             private_key_file: None,
+            certificate_database_file: None,
             transport_config: QuinnQuicTransportConfig::default(),
         }
     }
@@ -110,6 +115,7 @@ pub struct QuinnQuicSrc {
     settings: Mutex<Settings>,
     state: Mutex<State>,
     canceller: Mutex<utils::Canceller>,
+    connection: Mutex<Option<Connection>>,
 }
 
 impl Default for QuinnQuicSrc {
@@ -118,6 +124,7 @@ impl Default for QuinnQuicSrc {
             settings: Mutex::new(Settings::default()),
             state: Mutex::new(State::default()),
             canceller: Mutex::new(utils::Canceller::default()),
+            connection: Mutex::new(None),
         }
     }
 }
@@ -246,11 +253,15 @@ impl ObjectImpl for QuinnQuicSrc {
                     .build(),
                 glib::ParamSpecString::builder("certificate-file")
                     .nick("Certificate file")
-                    .blurb("Path to certificate chain in single file")
+                    .blurb("Path to certificate chain for the private key file in PEM format")
                     .build(),
                 glib::ParamSpecString::builder("private-key-file")
                     .nick("Private key file")
-                    .blurb("Path to a PKCS8 or RSA private key file")
+                    .blurb("Path to a PKCS1, PKCS8 or SEC1 private key file in PEM format")
+                    .build(),
+                glib::ParamSpecString::builder("certificate-database-file")
+                    .nick("Certificate database file")
+                    .blurb("Path to a certificate database file in PEM format used for certificate validation")
                     .build(),
                 glib::ParamSpecBoxed::builder::<gst::Caps>("caps")
                     .nick("caps")
@@ -349,11 +360,7 @@ impl ObjectImpl for QuinnQuicSrc {
                     .expect("type checked upstream")
                     .as_slice()
                     .iter()
-                    .map(|alpn| {
-                        alpn.get::<&str>()
-                            .expect("type checked upstream")
-                            .to_string()
-                    })
+                    .map(|alpn| alpn.get::<String>().expect("type checked upstream"))
                     .collect::<Vec<String>>()
             }
             "role" => {
@@ -365,8 +372,7 @@ impl ObjectImpl for QuinnQuicSrc {
                     .expect("type checked upstream")
                     .unwrap_or_else(gst::Caps::new_any);
 
-                let srcpad = self.obj().static_pad("src").expect("source pad expected");
-                srcpad.mark_reconfigure();
+                self.obj().src_pad().mark_reconfigure();
             }
             "timeout" => {
                 settings.timeout = value.get().expect("type checked upstream");
@@ -385,6 +391,10 @@ impl ObjectImpl for QuinnQuicSrc {
                 let value: String = value.get().unwrap();
                 settings.private_key_file = Some(value.into());
             }
+            "certificate-database-file" => {
+                let value: String = value.get().unwrap();
+                settings.certificate_database_file = Some(value.into());
+            }
             "use-datagram" => {
                 settings.use_datagram = value.get().expect("type checked upstream");
             }
@@ -438,12 +448,12 @@ impl ObjectImpl for QuinnQuicSrc {
 
         match pspec.name() {
             "server-name" => settings.server_name.to_value(),
-            "address" => settings.address.to_string().to_value(),
+            "address" => settings.address.to_value(),
             "port" => {
                 let port = settings.port as u32;
                 port.to_value()
             }
-            "bind-address" => settings.bind_address.to_string().to_value(),
+            "bind-address" => settings.bind_address.to_value(),
             "bind-port" => {
                 let port = settings.bind_port as u32;
                 port.to_value()
@@ -459,11 +469,15 @@ impl ObjectImpl for QuinnQuicSrc {
             "secure-connection" => settings.secure_conn.to_value(),
             "certificate-file" => {
                 let certfile = settings.certificate_file.as_ref();
-                certfile.and_then(|file| file.to_str()).to_value()
+                certfile.to_value()
             }
             "private-key-file" => {
                 let privkey = settings.private_key_file.as_ref();
-                privkey.and_then(|file| file.to_str()).to_value()
+                privkey.to_value()
+            }
+            "certificate-database-file" => {
+                let certfile = settings.certificate_database_file.as_ref();
+                certfile.to_value()
             }
             "use-datagram" => settings.use_datagram.to_value(),
             "initial-mtu" => (settings.transport_config.initial_mtu as u32).to_value(),
@@ -482,8 +496,7 @@ impl ObjectImpl for QuinnQuicSrc {
                 let state = self.state.lock().unwrap();
                 match *state {
                     State::Started(ref state) => {
-                        let connection = state.connection.clone();
-                        get_stats(Some(connection.stats())).to_value()
+                        get_stats(Some(state.connection.stats())).to_value()
                     }
                     State::Stopped => get_stats(None).to_value(),
                 }
@@ -509,61 +522,59 @@ impl ObjectSubclass for QuinnQuicSrc {
 
 impl BaseSrcImpl for QuinnQuicSrc {
     fn start(&self) -> Result<(), gst::ErrorMessage> {
-        let settings = self.settings.lock().unwrap();
-        let timeout = settings.timeout;
-        drop(settings);
-
         let state = self.state.lock().unwrap();
         if let State::Started { .. } = *state {
             unreachable!("QuicSrc already started");
         }
         drop(state);
 
-        match wait(&self.canceller, self.init_connection(), timeout) {
-            Ok(Ok(conn)) => {
-                let connection = conn.clone();
+        let (role, endpoint_config) = self.get_role_and_epconfig()?;
+        let socket_addr = self.get_socket_address(role, &endpoint_config);
 
-                let (tx_quit, rx_quit): (oneshot::Sender<()>, oneshot::Receiver<()>) =
-                    oneshot::channel();
-                let (data_tx, data_rx): (Sender<QuinnData>, Receiver<QuinnData>) = unbounded();
-
-                let conn_clone = conn.clone();
-                let self_ = self.ref_counted();
-                let data_handler = Builder::new()
-                    .name(DATA_HANDLER_THREAD.to_string())
-                    .spawn(move || {
-                        self_.handle_data(conn_clone, data_tx, rx_quit);
-                        gst::debug!(CAT, imp = self_, "Data handler thread exit");
-                    })
-                    .unwrap();
-
-                let mut state = self.state.lock().unwrap();
-                *state = State::Started(Started {
-                    connection,
-                    data_handler: Some(data_handler),
-                    data_rx: Some(data_rx),
-                    thread_quit: Some(tx_quit),
-                });
-                drop(state);
-
-                gst::info!(CAT, imp = self, "Started");
-
-                Ok(())
+        let conn_guard = self.connection.lock().unwrap();
+        let connection = match *conn_guard {
+            Some(ref c) => {
+                // We will end up here if downstream MoQ demuxer
+                // requested for a Connection before we could
+                // set up.
+                gst::info!(
+                    CAT,
+                    imp = self,
+                    "Using existing connection with ID: {}",
+                    c.stable_id()
+                );
+                c.clone()
             }
-            Ok(Err(e)) | Err(e) => match e {
-                WaitError::FutureAborted => {
-                    gst::warning!(CAT, imp = self, "Connection aborted");
-                    Ok(())
-                }
-                WaitError::FutureError(err) => {
-                    gst::error!(CAT, imp = self, "Connection request failed: {}", err);
-                    Err(gst::error_msg!(
-                        gst::ResourceError::Failed,
-                        ["Connection request failed: {}", err]
-                    ))
-                }
-            },
-        }
+            None => self.setup_connection(socket_addr, role, endpoint_config)?,
+        };
+        drop(conn_guard);
+
+        let (tx_quit, rx_quit): (oneshot::Sender<()>, oneshot::Receiver<()>) = oneshot::channel();
+        let (data_tx, data_rx): (Sender<QuinnData>, Receiver<QuinnData>) = unbounded();
+
+        let conn_clone = connection.clone();
+        let self_ = self.ref_counted();
+        let data_handler = Builder::new()
+            .name(DATA_HANDLER_THREAD.to_string())
+            .spawn(move || {
+                self_.handle_data(conn_clone, data_tx, rx_quit);
+                gst::debug!(CAT, imp = self_, "Data handler thread exit");
+            })
+            .unwrap();
+
+        let mut state = self.state.lock().unwrap();
+        *state = State::Started(Started {
+            connection,
+            data_handler: Some(data_handler),
+            data_rx: Some(data_rx),
+            thread_quit: Some(tx_quit),
+            socket_addr,
+        });
+        drop(state);
+
+        gst::info!(CAT, imp = self, "Started");
+
+        Ok(())
     }
 
     fn stop(&self) -> Result<(), gst::ErrorMessage> {
@@ -591,10 +602,14 @@ impl BaseSrcImpl for QuinnQuicSrc {
                 CONNECTION_CLOSE_CODE.into(),
                 CONNECTION_CLOSE_MSG.as_bytes(),
             );
+
+            SharedConnection::remove(state.socket_addr);
         }
 
         *state = State::Stopped;
 
+        *self.connection.lock().unwrap() = None;
+
         gst::info!(CAT, imp = self, "Stopped");
 
         Ok(())
@@ -640,6 +655,56 @@ impl BaseSrcImpl for QuinnQuicSrc {
     fn is_seekable(&self) -> bool {
         false
     }
+
+    fn query(&self, query: &mut gst::QueryRef) -> bool {
+        use gst::QueryViewMut;
+
+        match query.view_mut() {
+            QueryViewMut::Context(c) => {
+                gst::debug!(CAT, imp = self, "Handling Context query");
+
+                if let Some(context) = self.set_connection_context() {
+                    gst::info!(CAT, imp = self, "Setting Quinn Connection Context");
+                    c.set_context(&context);
+                    return true;
+                }
+
+                let (role, endpoint_config) = match self.get_role_and_epconfig() {
+                    Ok(config) => config,
+                    Err(err) => {
+                        gst::error!(CAT, imp = self, "Failed to get endpoint config: {}", err);
+                        return false;
+                    }
+                };
+                let addr = self.get_socket_address(role, &endpoint_config);
+
+                match self.setup_connection(addr, role, endpoint_config) {
+                    Ok(connection) => {
+                        let mut conn_guard = self.connection.lock().unwrap();
+                        *conn_guard = Some(connection);
+                        drop(conn_guard);
+
+                        match self.set_connection_context() {
+                            Some(context) => {
+                                gst::info!(CAT, imp = self, "Setting Quinn Connection Context");
+                                c.set_context(&context);
+                                true
+                            }
+                            None => {
+                                gst::error!(CAT, imp = self, "Failed to set context");
+                                false
+                            }
+                        }
+                    }
+                    Err(e) => {
+                        gst::error!(CAT, imp = self, "Failed to setup connection, {e:?}");
+                        false
+                    }
+                }
+            }
+            _ => BaseSrcImplExt::parent_query(self, query),
+        }
+    }
 }
 
 impl PushSrcImpl for QuinnQuicSrc {
@@ -676,8 +741,7 @@ impl PushSrcImpl for QuinnQuicSrc {
                 Ok(Some(QuinnData::Closed(stream_id))) => {
                     // Send custom downstream event for demuxer to close
                     // and remove the stream.
-                    let srcpad = self.obj().static_pad("src").expect("source pad expected");
-                    close_stream(&srcpad, stream_id);
+                    close_stream(self.obj().src_pad(), stream_id);
                 }
             }
         }
@@ -702,7 +766,7 @@ impl QuinnQuicSrc {
             };
         }
 
-        CreateSuccess::NewBuffer(buffer.to_owned())
+        CreateSuccess::NewBuffer(buffer)
     }
 
     fn get(&self) -> Result<Option<QuinnData>, Option<gst::ErrorMessage>> {
@@ -740,6 +804,45 @@ impl QuinnQuicSrc {
         }
     }
 
+    fn get_role_and_epconfig(
+        &self,
+    ) -> Result<(QuinnQuicRole, QuinnQuicEndpointConfig), gst::ErrorMessage> {
+        let settings = self.settings.lock().unwrap();
+
+        let client_addr =
+            make_socket_addr(format!("{}:{}", settings.bind_address, settings.bind_port).as_str())?;
+
+        let server_addr =
+            make_socket_addr(format!("{}:{}", settings.address, settings.port).as_str())?;
+
+        let server_name = settings.server_name.clone();
+        let alpns = settings.alpns.clone();
+        let role = settings.role;
+        let keep_alive_interval = settings.keep_alive_interval;
+        let secure_conn = settings.secure_conn;
+        let certificate_file = settings.certificate_file.clone();
+        let private_key_file = settings.private_key_file.clone();
+        let certificate_database_file = settings.certificate_database_file.clone();
+        let transport_config = settings.transport_config;
+
+        Ok((
+            role,
+            QuinnQuicEndpointConfig {
+                server_addr,
+                server_name,
+                client_addr: Some(client_addr),
+                secure_conn,
+                alpns,
+                certificate_file,
+                private_key_file,
+                certificate_database_file,
+                keep_alive_interval,
+                transport_config,
+                webtransport: false,
+            },
+        ))
+    }
+
     fn handle_connection_error(&self, connection_err: ConnectionError) {
         match connection_err {
             ConnectionError::ConnectionClosed(cc) => {
@@ -830,7 +933,7 @@ impl QuinnQuicSrc {
             }
         };
 
-        let tx_send = |sender: Sender<QuinnData>, data: QuinnData| async move {
+        let tx_send = |data: QuinnData| async {
             if let Err(err) = sender.send(data).await {
                 gst::error!(CAT, imp = self, "Error sending data: {err:?}");
             }
@@ -855,23 +958,23 @@ impl QuinnQuicSrc {
             while let Some(stream) = tasks.next().await {
                 match stream {
                     QuinnFuture::Stop => {
-                        tx_send(sender.clone(), QuinnData::Eos).await;
+                        tx_send(QuinnData::Eos).await;
                         break;
                     }
                     QuinnFuture::StreamData(s, data) => match data {
                         d @ QuinnData::Stream(stream_id, _) => {
                             gst::trace!(CAT, imp = self, "Sending data for stream: {stream_id}");
-                            tx_send(sender.clone(), d).await;
+                            tx_send(d).await;
                             tasks.push(Box::pin(recv_stream(s)));
                         }
                         eos @ QuinnData::Eos => {
-                            tx_send(sender.clone(), eos).await;
+                            tx_send(eos).await;
                             drop(s);
                             break;
                         }
                         c @ QuinnData::Closed(stream_id) => {
                             gst::trace!(CAT, imp = self, "Stream closed: {stream_id}");
-                            tx_send(sender.clone(), c).await;
+                            tx_send(c).await;
                             drop(s);
                         }
                         QuinnData::Datagram(_) => unreachable!(),
@@ -883,7 +986,7 @@ impl QuinnQuicSrc {
                     }
                     QuinnFuture::Datagram(b) => {
                         gst::trace!(CAT, imp = self, "Received {} bytes on datagram", b.len());
-                        tx_send(sender.clone(), QuinnData::Datagram(b)).await;
+                        tx_send(QuinnData::Datagram(b)).await;
                         tasks.push(Box::pin(datagram(connection.clone())));
                     }
                 }
@@ -893,88 +996,126 @@ impl QuinnQuicSrc {
         gst::info!(CAT, imp = self, "Quit data handler thread");
     }
 
-    async fn init_connection(&self) -> Result<Connection, WaitError> {
-        let (role, endpoint_config) = {
-            let settings = self.settings.lock().unwrap();
+    fn get_socket_address(
+        &self,
+        role: QuinnQuicRole,
+        endpoint_config: &QuinnQuicEndpointConfig,
+    ) -> SocketAddr {
+        match role {
+            QuinnQuicRole::Client => endpoint_config.server_addr,
+            QuinnQuicRole::Server => endpoint_config
+                .client_addr
+                .expect("Client address should be valid"),
+        }
+    }
 
-            let client_addr = make_socket_addr(
-                format!("{}:{}", settings.bind_address, settings.bind_port).as_str(),
-            )?;
+    fn setup_connection(
+        &self,
+        addr: SocketAddr,
+        role: QuinnQuicRole,
+        endpoint_config: QuinnQuicEndpointConfig,
+    ) -> Result<Connection, gst::ErrorMessage> {
+        let settings = self.settings.lock().unwrap();
+        let timeout = settings.timeout;
+        drop(settings);
 
-            let server_addr =
-                make_socket_addr(format!("{}:{}", settings.address, settings.port).as_str())?;
+        let mut shared_connection = SharedConnection::get_or_init(addr);
+        shared_connection.set_endpoint_config(endpoint_config);
 
-            let server_name = settings.server_name.clone();
-            let alpns = settings.alpns.clone();
-            let role = settings.role;
-            let keep_alive_interval = settings.keep_alive_interval;
-            let secure_conn = settings.secure_conn;
-            let certificate_file = settings.certificate_file.clone();
-            let private_key_file = settings.private_key_file.clone();
-            let transport_config = settings.transport_config;
+        let connection = match shared_connection.connection() {
+            Some(c) => match c {
+                QuinnConnection::Quic(conn) => {
+                    gst::info!(
+                        CAT,
+                        imp = self,
+                        "Using existing connection with ID: {}",
+                        conn.stable_id()
+                    );
 
-            (
-                role,
-                QuinnQuicEndpointConfig {
-                    server_addr,
-                    server_name,
-                    client_addr: Some(client_addr),
-                    secure_conn,
-                    alpns,
-                    certificate_file,
-                    private_key_file,
-                    keep_alive_interval,
-                    transport_config,
-                    with_client_auth: false,
+                    conn
+                }
+                QuinnConnection::WebTransport(_) => unreachable!(),
+            },
+            None => match wait(
+                &self.canceller,
+                shared_connection.connect(role, None),
+                timeout,
+            ) {
+                Ok(Ok(_)) => {
+                    let c = shared_connection
+                        .connection()
+                        .expect("Connection should be valid here");
+                    match c {
+                        QuinnConnection::WebTransport(_) => unreachable!(),
+                        QuinnConnection::Quic(conn) => {
+                            gst::info!(
+                                CAT,
+                                imp = self,
+                                "Using existing connection with ID: {}",
+                                conn.stable_id()
+                            );
+
+                            conn
+                        }
+                    }
+                }
+                Ok(Err(e)) | Err(e) => match e {
+                    WaitError::FutureAborted => {
+                        gst::warning!(CAT, imp = self, "Connection aborted");
+                        return Err(gst::error_msg!(
+                            gst::ResourceError::Failed,
+                            ["Connection request aborted"]
+                        ));
+                    }
+                    WaitError::FutureError(err) => {
+                        gst::error!(CAT, imp = self, "Connection request failed: {}", err);
+                        return Err(gst::error_msg!(
+                            gst::ResourceError::Failed,
+                            ["Connection request failed: {err}"]
+                        ));
+                    }
                 },
-            )
-        };
-
-        let endpoint = match role {
-            QuinnQuicRole::Server => server_endpoint(&endpoint_config).map_err(|err| {
-                WaitError::FutureError(gst::error_msg!(
-                    gst::ResourceError::Failed,
-                    ["Failed to configure endpoint: {}", err]
-                ))
-            })?,
-            QuinnQuicRole::Client => client_endpoint(&endpoint_config).map_err(|err| {
-                WaitError::FutureError(gst::error_msg!(
-                    gst::ResourceError::Failed,
-                    ["Failed to configure endpoint: {}", err]
-                ))
-            })?,
-        };
-
-        let connection = match role {
-            QuinnQuicRole::Server => {
-                let incoming_conn = endpoint.accept().await.unwrap();
-
-                incoming_conn.await.map_err(|err| {
-                    WaitError::FutureError(gst::error_msg!(
-                        gst::ResourceError::Failed,
-                        ["Connection error: {}", err]
-                    ))
-                })?
-            }
-            QuinnQuicRole::Client => endpoint
-                .connect(endpoint_config.server_addr, &endpoint_config.server_name)
-                .unwrap()
-                .await
-                .map_err(|err| {
-                    WaitError::FutureError(gst::error_msg!(
-                        gst::ResourceError::Failed,
-                        ["Connection error: {}", err]
-                    ))
-                })?,
+            },
         };
 
         gst::info!(
             CAT,
             imp = self,
-            "Remote connection accepted: {}",
-            connection.remote_address()
+            "QUIC connection established with ID: {}",
+            connection.stable_id()
         );
 
         Ok(connection)
     }
+
+    fn set_connection_context(&self) -> Option<gst::Context> {
+        let conn_guard = self.connection.lock().unwrap();
+
+        if let Some(ref connection) = *conn_guard {
+            gst::debug!(CAT, imp = self, "Using already established Connection");
+
+            let conn = QuinnConnectionContext(Arc::new(QuinnConnectionContextInner {
+                connection: QuinnConnection::Quic(connection.clone()),
+            }));
+
+            let mut context = gst::Context::new(QUINN_CONNECTION_CONTEXT, true);
+            {
+                let context = context.get_mut().unwrap();
+                let s = context.structure_mut();
+                s.set("connection", conn);
+            };
+
+            self.obj().set_context(&context);
+
+            let _ = self.obj().post_message(
+                gst::message::HaveContext::builder(context.clone())
+                    .src(&*self.obj())
+                    .build(),
+            );
+
+            return Some(context);
+        }
+
+        None
+    }
 }
