@@ -1,4 +1,4 @@
-use anyhow::{anyhow, bail, Context};
+use anyhow::{Context, anyhow, bail};
 use async_tungstenite::tungstenite::Message;
 use futures::prelude::*;
 use futures::{pin_mut, select, select_biased};
@@ -66,11 +66,7 @@ struct Args {
 
 impl Args {
     pub fn scheme(&self) -> &str {
-        if self.use_tls {
-            "wss"
-        } else {
-            "ws"
-        }
+        if self.use_tls { "wss" } else { "ws" }
     }
 
     async fn get_synced_clock(&self) -> anyhow::Result<gst::Clock> {
@@ -132,8 +128,8 @@ fn spawn_consumer(
         // * https://gitlab.freedesktop.org/gstreamer/gst-plugins-good/-/issues/914
         // * https://gitlab.freedesktop.org/gstreamer/gstreamer/-/issues/1574
         .property_if("do-retransmission", false, args.expect_clock_signalling)
-        .property_if_not_empty::<gst::Array>("audio-codecs", &args.audio_codecs)
-        .property_if_not_empty::<gst::Array>("video-codecs", &args.video_codecs)
+        .property_if_not_empty::<gst::Array, _>("audio-codecs", &args.audio_codecs)
+        .property_if_not_empty::<gst::Array, _>("video-codecs", &args.video_codecs)
         .build()
         .context("Creating webrtcsrc")?;
 
@@ -508,7 +504,11 @@ impl App {
                                 "Latency requirements have changed for element {}",
                                 msg.src().map(|src| src.name()).as_deref().unwrap_or("UNKNOWN"),
                             );
-                            if let Err(err) = self.pipeline().recalculate_latency() {
+                            if let Err(err) = self
+                                .pipeline()
+                                .call_async_future(|pipeline| pipeline.recalculate_latency())
+                                .await
+                            {
                                 error!(%err, "Error recalculating latency");
                             }
                         }
@@ -556,8 +556,8 @@ impl App {
 async fn connect_as_listener(
     signaller_url: &Url,
 ) -> anyhow::Result<(
-    Pin<Box<impl Sink<ToSignaller, Error = anyhow::Error>>>,
-    Pin<Box<impl Stream<Item = anyhow::Result<FromSignaller>>>>,
+    Pin<Box<impl Sink<ToSignaller, Error = anyhow::Error> + use<>>>,
+    Pin<Box<impl Stream<Item = anyhow::Result<FromSignaller>> + use<>>>,
 )> {
     async fn register(
         mut signaller_tx: Pin<&mut impl Sink<ToSignaller, Error = anyhow::Error>>,
@@ -735,10 +735,10 @@ async fn listener_task(
     debug!("Closing signaller websocket");
     let _ = signaller_tx.close().await;
 
-    if let Ok(listener_res) = res {
-        if listener_res.is_err() {
-            listener_res?;
-        }
+    if let Ok(listener_res) = res
+        && listener_res.is_err()
+    {
+        listener_res?;
     }
 
     Ok(())
