@@ -1,18 +1,18 @@
 // SPDX-License-Identifier: MPL-2.0
 
 use crate::{
+    RUNTIME,
     signaller::{Signallable, SignallableImpl, WebRTCSignallerRole},
     webrtcsink::JanusVRSignallerState,
-    RUNTIME,
 };
 
-use anyhow::{anyhow, Error};
+use anyhow::{Error, anyhow};
 use async_tungstenite::tungstenite;
 use futures::channel::mpsc;
 use futures::sink::SinkExt;
 use futures::stream::StreamExt;
 use gst::glib;
-use gst::glib::{subclass::Signal, Properties};
+use gst::glib::{Properties, subclass::Signal};
 use gst::prelude::*;
 use gst::subclass::prelude::*;
 use http::Uri;
@@ -347,7 +347,7 @@ impl Signaller {
         .await??;
 
         // Channel for asynchronously sending out websocket message
-        let (ws_sink, mut ws_stream) = ws.split();
+        let (mut ws_sink, mut ws_stream) = ws.split();
 
         // 1000 is completely arbitrary, we simply don't want infinite piling
         // up of messages as with unbounded
@@ -941,10 +941,13 @@ impl SignallableImpl for Signaller {
         let imp = self.downgrade();
         RUNTIME.spawn(async move {
             if let Some(imp) = imp.upgrade() {
-                if let Err(err) = imp.connect().await {
-                    this.emit_by_name::<()>("error", &[&format!("{:?}", anyhow!(err))]);
-                } else {
-                    imp.create_session();
+                match imp.connect().await {
+                    Err(err) => {
+                        this.emit_by_name::<()>("error", &[&format!("{:?}", anyhow!(err))]);
+                    }
+                    _ => {
+                        imp.create_session();
+                    }
                 }
             }
         });
@@ -1006,10 +1009,10 @@ impl SignallableImpl for Signaller {
             RUNTIME.block_on(async move {
                 sender.close_channel();
 
-                if let Some(handle) = send_task_handle {
-                    if let Err(err) = handle.await {
-                        gst::warning!(CAT, imp = self, "Error while joining send task: {}", err);
-                    }
+                if let Some(handle) = send_task_handle
+                    && let Err(err) = handle.await
+                {
+                    gst::warning!(CAT, imp = self, "Error while joining send task: {}", err);
                 }
 
                 if let Some(handle) = recv_task_handle {
