@@ -1,17 +1,17 @@
 // SPDX-License-Identifier: MPL-2.0
 
+use crate::RUNTIME;
 use crate::signaller::{Signallable, SignallableExt, SignallableImpl};
 use crate::utils::{
-    build_link_header, build_reqwest_client, parse_redirect_location, set_ice_servers, wait,
-    wait_async, WaitError,
+    WaitError, build_link_header, build_reqwest_client, parse_redirect_location, set_ice_servers,
+    wait, wait_async,
 };
-use crate::RUNTIME;
-use async_recursion::async_recursion;
 use gst::glib::{self, RustClosure};
 use gst::prelude::*;
 use gst::subclass::prelude::*;
 use gst_sdp::SDPMessage;
 use gst_webrtc::{WebRTCICEGatheringState, WebRTCSessionDescription};
+use http::{HeaderName, HeaderValue};
 use std::collections::HashMap;
 use std::sync::LazyLock;
 use std::sync::Mutex;
@@ -19,7 +19,7 @@ use std::sync::Mutex;
 use std::net::SocketAddr;
 use tokio::sync::mpsc;
 use url::Url;
-use warp::{http, hyper::Body, Filter, Reply};
+use warp::{Filter, Reply, http};
 
 static CAT: LazyLock<gst::DebugCategory> = LazyLock::new(|| {
     gst::DebugCategory::new(
@@ -40,17 +40,16 @@ const DEFAULT_STUN_SERVER: Option<&str> = Some("stun://stun.l.google.com:19303")
 const CONTENT_SDP: &str = "application/sdp";
 const CONTENT_TRICKLE_ICE: &str = "application/trickle-ice-sdpfrag";
 
-#[derive(Debug)]
+#[derive(Debug, Default)]
 enum WhipClientState {
+    #[default]
     Stopped,
-    Post { redirects: u8 },
-    Running { whip_resource_url: String },
-}
-
-impl Default for WhipClientState {
-    fn default() -> Self {
-        Self::Stopped
-    }
+    Post {
+        redirects: u8,
+    },
+    Running {
+        whip_resource_url: String,
+    },
 }
 
 #[derive(Clone)]
@@ -140,7 +139,6 @@ impl WhipClient {
         }
     }
 
-    #[async_recursion]
     async fn do_post(
         &self,
         offer: gst_webrtc::WebRTCSessionDescription,
@@ -234,12 +232,10 @@ impl WhipClient {
 
         match resp.status() {
             reqwest::StatusCode::OK | reqwest::StatusCode::CREATED => {
-                if use_link_headers {
-                    if let Err(e) = set_ice_servers(webrtcbin, resp.headers()) {
-                        self.raise_error(e.to_string());
-                        return;
-                    };
-                }
+                if use_link_headers && let Err(e) = set_ice_servers(webrtcbin, resp.headers()) {
+                    self.raise_error(e.to_string());
+                    return;
+                };
 
                 // Get the url of the resource from 'location' header.
                 // The resource created is expected be a relative path
@@ -348,7 +344,7 @@ impl WhipClient {
                                 redirect_url.as_str()
                             );
 
-                            self.do_post(offer, webrtcbin, redirect_url).await
+                            Box::pin(self.do_post(offer, webrtcbin, redirect_url)).await
                         }
                         Err(e) => self.raise_error(e.to_string()),
                     }
@@ -428,6 +424,7 @@ impl WhipClient {
                 }
             },
         };
+        self.obj().emit_by_name::<()>("shutdown", &[])
     }
 }
 
@@ -618,7 +615,7 @@ impl ObjectImpl for WhipClient {
 struct WhipServerSettings {
     stun_server: Option<String>,
     turn_servers: gst::Array,
-    host_addr: Url,
+    host_addr: Option<Url>,
     timeout: u32,
     shutdown_signal: Option<tokio::sync::oneshot::Sender<()>>,
     server_handle: Option<tokio::task::JoinHandle<()>>,
@@ -628,7 +625,7 @@ struct WhipServerSettings {
 impl Default for WhipServerSettings {
     fn default() -> Self {
         Self {
-            host_addr: Url::parse(DEFAULT_HOST_ADDR).unwrap(),
+            host_addr: Some(Url::parse(DEFAULT_HOST_ADDR).unwrap()),
             stun_server: DEFAULT_STUN_SERVER.map(String::from),
             turn_servers: gst::Array::new(Vec::new() as Vec<glib::SendValue>),
             timeout: DEFAULT_TIMEOUT,
@@ -672,14 +669,15 @@ impl WhipServer {
                                 );
                                 let ans: Option<gst_sdp::SDPMessage>;
                                 let mut settings = signaller.imp().settings.lock().unwrap();
-                                if let Some(answer_desc) = webrtcbin
-                                    .property::<Option<WebRTCSessionDescription>>(
-                                        "local-description",
-                                    )
-                                {
-                                    ans = Some(answer_desc.sdp().to_owned());
-                                } else {
-                                    ans = None;
+                                match webrtcbin.property::<Option<WebRTCSessionDescription>>(
+                                    "local-description",
+                                ) {
+                                    Some(answer_desc) => {
+                                        ans = Some(answer_desc.sdp().to_owned());
+                                    }
+                                    _ => {
+                                        ans = None;
+                                    }
                                 }
                                 let tx = settings
                                     .sdp_answer
@@ -708,7 +706,10 @@ impl WhipServer {
         })
     }
 
-    async fn patch_handler(&self, _id: String) -> Result<impl warp::Reply, warp::Rejection> {
+    async fn patch_handler(
+        &self,
+        _id: String,
+    ) -> Result<impl warp::Reply + use<>, warp::Rejection> {
         // FIXME: implement ICE Trickle and ICE restart
         // emit signal `handle-ice` to for ICE trickle
         let reply = warp::reply::reply();
@@ -718,7 +719,7 @@ impl WhipServer {
         //FIXME: add state checking once ICE trickle is implemented
     }
 
-    async fn delete_handler(&self, id: String) -> Result<impl warp::Reply, warp::Rejection> {
+    async fn delete_session(&self, id: String) -> http::StatusCode {
         if self
             .obj()
             .emit_by_name::<bool>("session-ended", &[&id.as_str()])
@@ -728,11 +729,19 @@ impl WhipServer {
             gst::info!(CAT, imp = self, "Failed to End session {id}");
             // FIXME: Do we send a different response
         }
-        Ok(warp::reply::reply().into_response())
+
+        http::StatusCode::OK
+    }
+
+    async fn delete_handler(
+        &self,
+        id: String,
+    ) -> Result<impl warp::Reply + use<>, warp::Rejection> {
+        Ok(self.delete_session(id).await.into_response())
     }
 
     #[allow(clippy::single_match)]
-    async fn options_handler(&self) -> Result<impl warp::Reply, warp::Rejection> {
+    async fn options_handler(&self) -> Result<impl warp::reply::Reply + use<>, warp::Rejection> {
         let settings = self.settings.lock().unwrap();
         drop(settings);
 
@@ -780,7 +789,7 @@ impl WhipServer {
 
         let mut res = http::Response::builder()
             .header("Access-Post", CONTENT_SDP)
-            .body(Body::empty())
+            .body(bytes::Bytes::new())
             .unwrap();
 
         let headers = res.headers_mut();
@@ -789,11 +798,18 @@ impl WhipServer {
         Ok(res)
     }
 
-    async fn post_handler(
+    async fn create_session(
         &self,
-        body: warp::hyper::body::Bytes,
+        body: &[u8],
         id: Option<String>,
-    ) -> Result<http::Response<warp::hyper::Body>, warp::Rejection> {
+    ) -> (
+        http::StatusCode,
+        Vec<(HeaderName, HeaderValue)>,
+        Option<bytes::Bytes>,
+    ) {
+        let mut status = http::StatusCode::CREATED;
+        let mut headermap = Vec::new();
+
         let session_id = match id {
             Some(id) => {
                 gst::debug!(CAT, imp = self, "got session id {id} from the URL");
@@ -814,7 +830,7 @@ impl WhipServer {
             wait_timeout
         };
 
-        match gst_sdp::SDPMessage::parse_buffer(body.as_ref()) {
+        match gst_sdp::SDPMessage::parse_buffer(body) {
             Ok(offer_sdp) => {
                 let offer = gst_webrtc::WebRTCSessionDescription::new(
                     gst_webrtc::WebRTCSDPType::Offer,
@@ -828,9 +844,8 @@ impl WhipServer {
             }
             Err(err) => {
                 gst::error!(CAT, imp = self, "Could not parse offer SDP: {err}");
-                let reply = warp::reply::reply();
-                let res = warp::reply::with_status(reply, http::StatusCode::NOT_ACCEPTABLE);
-                return Ok(res.into_response());
+                status = http::StatusCode::NOT_ACCEPTABLE;
+                return (status, headermap, None);
             }
         }
 
@@ -844,12 +859,8 @@ impl WhipServer {
                 Some(a) => a,
                 None => {
                     let err = "Channel closed, can't receive SDP".to_owned();
-                    let res = http::Response::builder()
-                        .status(http::StatusCode::INTERNAL_SERVER_ERROR)
-                        .body(Body::from(err))
-                        .unwrap();
-
-                    return Ok(res);
+                    status = http::StatusCode::INTERNAL_SERVER_ERROR;
+                    return (status, headermap, Some(err.into()));
                 }
             },
             Err(e) => {
@@ -857,26 +868,22 @@ impl WhipServer {
                     WaitError::FutureAborted => "Aborted".to_owned(),
                     WaitError::FutureError(err) => err.to_string(),
                 };
-                let res = http::Response::builder()
-                    .status(http::StatusCode::INTERNAL_SERVER_ERROR)
-                    .body(Body::from(err))
-                    .unwrap();
-
-                return Ok(res);
+                status = http::StatusCode::INTERNAL_SERVER_ERROR;
+                return (status, headermap, Some(err.into()));
             }
         };
 
         let settings = self.settings.lock().unwrap();
-        let mut links = http::HeaderMap::new();
+        let mut links = vec![];
 
         #[allow(clippy::single_match)]
         match &settings.stun_server {
             Some(stun) => match build_link_header(stun.as_str()) {
                 Ok(stun_link) => {
-                    links.append(
+                    links.push((
                         http::header::LINK,
                         http::HeaderValue::from_str(stun_link.as_str()).unwrap(),
-                    );
+                    ));
                 }
                 Err(e) => {
                     gst::error!(CAT, imp = self, "Failed to parse {stun:?} : {e:?}");
@@ -891,10 +898,10 @@ impl WhipServer {
                     gst::debug!(CAT, imp = self, "turn server: {}", turn.as_str());
                     match build_link_header(turn.as_str()) {
                         Ok(turn_link) => {
-                            links.append(
+                            links.push((
                                 http::header::LINK,
                                 http::HeaderValue::from_str(turn_link.as_str()).unwrap(),
-                            );
+                            ));
                         }
                         Err(e) => {
                             gst::error!(
@@ -937,34 +944,50 @@ impl WhipServer {
 
         // If ans_text is an error. Send error code and error string in the response
         if let Err(e) = ans_text {
-            let res = http::Response::builder()
-                .status(http::StatusCode::INTERNAL_SERVER_ERROR)
-                .body(Body::from(e))
-                .unwrap();
-            return Ok(res);
+            status = http::StatusCode::INTERNAL_SERVER_ERROR;
+            return (status, headermap, Some(e.into()));
         }
 
         drop(settings);
 
-        // Got SDP answer, send answer in the response
         let resource_url = "/".to_owned() + ROOT + "/" + RESOURCE_PATH + "/" + &session_id;
-        let mut res = http::Response::builder()
-            .status(http::StatusCode::CREATED)
-            .header(http::header::CONTENT_TYPE, CONTENT_SDP)
-            .header("location", resource_url)
-            .body(Body::from(ans_text.unwrap()))
-            .unwrap();
 
-        let headers = res.headers_mut();
-        headers.extend(links);
+        headermap.push((
+            http::header::CONTENT_TYPE,
+            HeaderValue::from_str(CONTENT_SDP).unwrap(),
+        ));
+        headermap.push((
+            HeaderName::from_static("location"),
+            HeaderValue::from_str(resource_url.as_str()).unwrap(),
+        ));
+        headermap.append(&mut links);
 
-        Ok(res)
+        (status, headermap, Some(ans_text.unwrap().into()))
+    }
+
+    async fn post_handler(
+        &self,
+        body: &[u8],
+        id: Option<String>,
+    ) -> Result<impl warp::reply::Reply + use<>, warp::Rejection> {
+        let (status, mut headermap, body) = self.create_session(body, id.clone()).await;
+
+        let mut response_builder = http::Response::builder().status(status);
+
+        for (key, value) in headermap.drain(..) {
+            response_builder = response_builder.header(key, value);
+        }
+
+        Ok(response_builder.body(body).unwrap())
     }
 
     fn serve(&self) -> Option<tokio::task::JoinHandle<()>> {
         let mut settings = self.settings.lock().unwrap();
+
+        let host_addr = settings.host_addr.as_ref()?;
+
         let addr: SocketAddr;
-        match settings.host_addr.socket_addrs(|| None) {
+        match host_addr.socket_addrs(|| None) {
             Ok(v) => {
                 // pick the first vector item
                 addr = v[0];
@@ -982,6 +1005,49 @@ impl WhipServer {
         settings.shutdown_signal = Some(tx);
         drop(settings);
 
+        let api = self.filter();
+
+        let jh = RUNTIME.spawn(async move {
+            warp::serve(api)
+                .bind(addr)
+                .await
+                .graceful(async move {
+                    match rx.await {
+                        Ok(_) => gst::debug!(CAT, "Server shut down signal received"),
+                        Err(e) => gst::error!(CAT, "{e:?}: Sender dropped"),
+                    }
+                })
+                .run()
+                .await;
+
+            gst::debug!(CAT, "Stopped the server task...");
+        });
+
+        gst::debug!(CAT, imp = self, "Started the server...");
+        Some(jh)
+    }
+
+    fn set_host_addr(&self, host_addr: Option<&str>) -> Result<(), url::ParseError> {
+        let mut settings = self.settings.lock().unwrap();
+
+        if let Some(host_addr) = host_addr {
+            if !host_addr.is_empty() {
+                settings.host_addr = Some(Url::parse(host_addr)?);
+            } else {
+                // Special case for easily disabling server from gst-launch
+                settings.host_addr = None;
+            }
+        } else {
+            settings.host_addr = None;
+        }
+
+        Ok(())
+    }
+
+    fn filter(
+        &self,
+    ) -> impl Filter<Extract = impl warp::Reply + use<>> + Clone + Send + Sync + 'static + use<>
+    {
         let prefix = warp::path(ROOT);
 
         // POST /endpoint
@@ -997,7 +1063,9 @@ impl WhipServer {
                 #[weak(rename_to = self_)]
                 self,
                 #[upgrade_or_panic]
-                move |body| async move { self_.post_handler(body, None).await }
+                move |body: warp::hyper::body::Bytes| async move {
+                    self_.post_handler(body.as_ref(), None).await
+                }
             ));
 
         // POST /endpoint/<session-id>
@@ -1014,7 +1082,9 @@ impl WhipServer {
                 #[weak(rename_to = self_)]
                 self,
                 #[upgrade_or_panic]
-                move |id, body| async move { self_.post_handler(body, Some(id)).await }
+                move |id, body: warp::hyper::body::Bytes| async move {
+                    self_.post_handler(body.as_ref(), Some(id)).await
+                }
             ));
 
         // OPTIONS /endpoint
@@ -1056,33 +1126,11 @@ impl WhipServer {
                 move |id| async move { self_.delete_handler(id).await }
             ));
 
-        let api = prefix
+        prefix
             .and(post_filter.or(post_filter_with_id))
             .or(prefix.and(options_filter))
             .or(prefix.and(patch_filter))
-            .or(prefix.and(delete_filter));
-
-        let s = warp::serve(api);
-        let jh = RUNTIME.spawn(async move {
-            let (_, server) = s.bind_with_graceful_shutdown(addr, async move {
-                match rx.await {
-                    Ok(_) => gst::debug!(CAT, "Server shut down signal received"),
-                    Err(e) => gst::error!(CAT, "{e:?}: Sender dropped"),
-                }
-            });
-
-            server.await;
-            gst::debug!(CAT, "Stopped the server task...");
-        });
-
-        gst::debug!(CAT, imp = self, "Started the server...");
-        Some(jh)
-    }
-
-    fn set_host_addr(&self, host_addr: &str) -> Result<(), url::ParseError> {
-        let mut settings = self.settings.lock().unwrap();
-        settings.host_addr = Url::parse(host_addr)?;
-        Ok(())
+            .or(prefix.and(delete_filter))
     }
 }
 
@@ -1097,17 +1145,11 @@ impl SignallableImpl for WhipServer {
     fn stop(&self) {
         let mut settings = self.settings.lock().unwrap();
 
-        let handle = settings
-            .server_handle
-            .take()
-            .expect("Server handle should be set");
+        let handle = settings.server_handle.take();
 
-        let tx = settings
-            .shutdown_signal
-            .take()
-            .expect("Shutdown signal Sender needs to be valid");
-
-        if tx.send(()).is_err() {
+        if let Some(tx) = settings.shutdown_signal.take()
+            && tx.send(()).is_err()
+        {
             gst::error!(
                 CAT,
                 imp = self,
@@ -1115,14 +1157,16 @@ impl SignallableImpl for WhipServer {
             );
         }
 
-        gst::debug!(CAT, imp = self, "Await server handle to join");
-        RUNTIME.block_on(async {
-            if let Err(e) = handle.await {
-                gst::error!(CAT, imp = self, "Failed to join server handle: {e:?}");
-            };
-        });
+        if let Some(handle) = handle {
+            gst::debug!(CAT, imp = self, "Await server handle to join");
+            RUNTIME.block_on(async {
+                if let Err(e) = handle.await {
+                    gst::error!(CAT, imp = self, "Failed to join server handle: {e:?}");
+                };
+            });
 
-        gst::info!(CAT, imp = self, "stopped the WHIP server");
+            gst::info!(CAT, imp = self, "stopped the WHIP server");
+        }
     }
 
     fn end_session(&self, session_id: &str) {
@@ -1185,9 +1229,12 @@ impl ObjectImpl for WhipServer {
         match pspec.name() {
             "host-addr" => {
                 if let Err(e) =
-                    self.set_host_addr(value.get::<&str>().expect("type checked upstream"))
+                    self.set_host_addr(value.get::<Option<&str>>().expect("type checked upstream"))
                 {
-                    gst::error!(CAT, "Couldn't set the host address as {e:?}, fallback to the default value {DEFAULT_HOST_ADDR:?}");
+                    gst::error!(
+                        CAT,
+                        "Couldn't set the host address as {e:?}, fallback to the default value {DEFAULT_HOST_ADDR:?}"
+                    );
                 }
             }
             "stun-server" => {
@@ -1212,11 +1259,142 @@ impl ObjectImpl for WhipServer {
         let settings = self.settings.lock().unwrap();
         match pspec.name() {
             "manual-sdp-munging" => false.to_value(),
-            "host-addr" => settings.host_addr.to_string().to_value(),
+            "host-addr" => settings
+                .host_addr
+                .as_ref()
+                .map(|host_addr| host_addr.to_string())
+                .to_value(),
             "stun-server" => settings.stun_server.to_value(),
             "turn-servers" => settings.turn_servers.to_value(),
             "timeout" => settings.timeout.to_value(),
             _ => unimplemented!(),
         }
     }
+
+    fn signals() -> &'static [glib::subclass::Signal] {
+        static SIGNALS: LazyLock<Vec<glib::subclass::Signal>> = LazyLock::new(|| {
+            vec![
+                /**
+                 * GstWhipServerSignaller::post:
+                 * @self: a signaller instance
+                 * @id: optional session identifier
+                 * @request_body: body of the POST request
+                 * @promise: promise to receive the result
+                 *
+                 * When using whipserversrc with an external http server, use
+                 * this action signal to forward POST requests to the signaller.
+                 *
+                 * The #GstStructure in the promise reply will contain:
+                 *
+                 * * A "status" field with type #guint32
+                 * * A "headers" field with type #GstStructure with all string values
+                 * * An optional "body" field with type #GBytes
+                 */
+                glib::subclass::Signal::builder("post")
+                    .action()
+                    .class_handler(|args| {
+                        let signaller = args[0]
+                            .get::<super::WhipServerSignaller>()
+                            .expect("signal arg");
+                        let id = args[1]
+                            .get::<Option<String>>()
+                            .expect("ID as string as first parameter");
+                        let request_body = args[2]
+                            .get::<glib::Bytes>()
+                            .expect("request body as GBytes as second parameter");
+                        let promise = args[3]
+                            .get::<gst::Promise>()
+                            .expect("GstPromise as third parameter");
+
+                        RUNTIME.spawn(async move {
+                            let (status, mut headers, body) =
+                                signaller.imp().create_session(&request_body, id).await;
+
+                            let mut headers_builder =
+                                gst::Structure::builder("whip-wignaller/headers");
+
+                            for (header, value) in headers.drain(..) {
+                                let value = value.to_str().expect(
+                                    "Header value should contain only visible ASCII strings",
+                                );
+                                headers_builder = headers_builder.field(header.to_string(), value);
+                            }
+
+                            let mut reply_builder =
+                                gst::Structure::builder("whip-signaller/response")
+                                    .field("status", status.as_u16() as u32)
+                                    .field("headers", headers_builder.build());
+
+                            if let Some(body) = body {
+                                reply_builder =
+                                    reply_builder.field("body", glib::Bytes::from(&body));
+                            }
+
+                            let reply = reply_builder.build();
+
+                            gst::log!(CAT, obj = signaller, "replying to promise with {reply:?}");
+
+                            promise.reply(Some(reply));
+                        });
+
+                        None
+                    })
+                    .param_types([
+                        String::static_type(),
+                        glib::Bytes::static_type(),
+                        gst::Promise::static_type(),
+                    ])
+                    .return_type::<()>()
+                    .build(),
+                /**
+                 * GstWhipServerSignaller::delete:
+                 * @self: a signaller instance
+                 * @id: optional session identifier
+                 * @promise: promise to receive the result
+                 *
+                 * When using whipserversrc with an external http server, use
+                 * this action signal to forward DELETE requests to the signaller.
+                 *
+                 * The #GstStructure in the promise reply will contain:
+                 *
+                 * * A "status" field with type #guint32
+                 */
+                glib::subclass::Signal::builder("delete")
+                    .action()
+                    .class_handler(|args| {
+                        let signaller = args[0]
+                            .get::<super::WhipServerSignaller>()
+                            .expect("signal arg");
+                        let id = args[1]
+                            .get::<String>()
+                            .expect("ID as string as first parameter");
+                        let promise = args[2]
+                            .get::<gst::Promise>()
+                            .expect("GstPromise as third parameter");
+
+                        RUNTIME.spawn(async move {
+                            let status = signaller.imp().delete_session(id).await;
+
+                            let reply = gst::Structure::builder("whip-signaller/response")
+                                .field("status", status.as_u16() as u32)
+                                .build();
+
+                            gst::log!(CAT, obj = signaller, "replying to promise with {reply:?}");
+
+                            promise.reply(Some(reply));
+                        });
+
+                        None
+                    })
+                    .param_types([
+                        String::static_type(),
+                        gst::Promise::static_type(),
+                    ])
+                    .return_type::<()>()
+                    .build(),
+            ]
+        });
+
+        SIGNALS.as_ref()
+    }
 }
