@@ -197,11 +197,7 @@ trait HandleData: Sized {
     fn dts(&self) -> Option<gst::ClockTime>;
     fn dts_or_pts(&self) -> Option<gst::ClockTime> {
         let dts = self.dts();
-        if dts.is_some() {
-            dts
-        } else {
-            self.pts()
-        }
+        if dts.is_some() { dts } else { self.pts() }
     }
     fn duration(&self, state: &StreamState) -> Option<gst::ClockTime>;
     fn is_keyframe(&self) -> bool;
@@ -384,9 +380,7 @@ impl ToggleRecord {
             let mut settings = self.settings.lock();
 
             if rec_state.time_start_block.is_none() {
-                rec_state.time_start_block = clock
-                    .as_ref()
-                    .map_or(state.current_running_time, |c| c.time());
+                rec_state.time_start_block = clock.as_ref().map(gst::Clock::time);
             }
             while !settings.record && !state.flushing {
                 gst::debug!(CAT, obj = pad, "Waiting for record=true");
@@ -411,7 +405,7 @@ impl ToggleRecord {
             if let Some(time_start_block) = rec_state.time_start_block {
                 // If we have a time_start_block it means the clock is there
                 let clock = clock.expect("Cannot find pipeline clock");
-                rec_state.blocked_duration += clock.time().unwrap() - time_start_block;
+                rec_state.blocked_duration += clock.time() - time_start_block;
                 if settings.live {
                     rec_state.running_time_offset = rec_state.blocked_duration.nseconds() as i64;
                 }
@@ -505,7 +499,7 @@ impl ToggleRecord {
         let settings_changed = match rec_state.recording_state {
             RecordingState::Recording if !settings.record => {
                 let clock = self.obj().clock().expect("Cannot find pipeline clock");
-                rec_state.time_start_block = Some(clock.time().unwrap());
+                rec_state.time_start_block = Some(clock.time());
                 gst::debug!(CAT, obj = pad, "Stopping recording");
                 rec_state.recording_state = RecordingState::Stopping;
                 true
@@ -920,11 +914,14 @@ impl ToggleRecord {
 
                 gst::log!(CAT, obj = pad, "Clipping to segment {:?}", segment);
 
-                if let Some(data) = data.clip(&state, &segment) {
-                    return Ok(HandleResult::Pass(data));
-                } else {
-                    gst::warning!(CAT, obj = pad, "Complete buffer clipped!");
-                    return Ok(HandleResult::Drop);
+                match data.clip(&state, &segment) {
+                    Some(data) => {
+                        return Ok(HandleResult::Pass(data));
+                    }
+                    _ => {
+                        gst::warning!(CAT, obj = pad, "Complete buffer clipped!");
+                        return Ok(HandleResult::Drop);
+                    }
                 }
             } else if current_running_time
                 .opt_lt(last_recording_start)
@@ -978,16 +975,19 @@ impl ToggleRecord {
 
                 gst::log!(CAT, obj = pad, "Clipping to segment {:?}", segment,);
 
-                if let Some(data) = data.clip(&state, &segment) {
-                    return Ok(HandleResult::Pass(data));
-                } else {
-                    gst::warning!(CAT, obj = pad, "Complete buffer clipped!");
-                    return Ok(HandleResult::Eos(self.check_and_update_eos(
-                        pad,
-                        stream,
-                        &mut state,
-                        &mut rec_state,
-                    )));
+                match data.clip(&state, &segment) {
+                    Some(data) => {
+                        return Ok(HandleResult::Pass(data));
+                    }
+                    _ => {
+                        gst::warning!(CAT, obj = pad, "Complete buffer clipped!");
+                        return Ok(HandleResult::Eos(self.check_and_update_eos(
+                            pad,
+                            stream,
+                            &mut state,
+                            &mut rec_state,
+                        )));
+                    }
                 }
             } else if current_running_time_end
                 .opt_gt(rec_state.last_recording_stop)
@@ -1012,12 +1012,16 @@ impl ToggleRecord {
             } else {
                 // In all other cases the buffer is fully between recording start and end and
                 // can be passed through as is
-                assert!(current_running_time
-                    .opt_ge(last_recording_start)
-                    .unwrap_or(false));
-                assert!(current_running_time_end
-                    .opt_le(rec_state.last_recording_stop)
-                    .unwrap_or(false));
+                assert!(
+                    current_running_time
+                        .opt_ge(last_recording_start)
+                        .unwrap_or(false)
+                );
+                assert!(
+                    current_running_time_end
+                        .opt_le(rec_state.last_recording_stop)
+                        .unwrap_or(false)
+                );
 
                 gst::debug!(
                     CAT,
@@ -1039,9 +1043,11 @@ impl ToggleRecord {
             RecordingState::Recording => {
                 // The end of our buffer must be before/at the end of the previous buffer of the main
                 // stream
-                assert!(current_running_time_end
-                    .opt_le(main_state.current_running_time_end)
-                    .unwrap_or(false));
+                assert!(
+                    current_running_time_end
+                        .opt_le(main_state.current_running_time_end)
+                        .unwrap_or(false)
+                );
 
                 // We're properly started, must have a start position and
                 // be actually after that start position
@@ -1112,11 +1118,12 @@ impl ToggleRecord {
 
                     gst::log!(CAT, obj = pad, "Clipping to segment {:?}", segment,);
 
-                    if let Some(data) = data.clip(&state, &segment) {
-                        Ok(HandleResult::Pass(data))
-                    } else {
-                        gst::warning!(CAT, obj = pad, "Complete buffer clipped!");
-                        Ok(HandleResult::Drop)
+                    match data.clip(&state, &segment) {
+                        Some(data) => Ok(HandleResult::Pass(data)),
+                        _ => {
+                            gst::warning!(CAT, obj = pad, "Complete buffer clipped!");
+                            Ok(HandleResult::Drop)
+                        }
                     }
                 } else {
                     gst::log!(
@@ -1132,9 +1139,11 @@ impl ToggleRecord {
             RecordingState::Stopped => {
                 // The end of our buffer must be before/at the end of the previous buffer of the main
                 // stream
-                assert!(current_running_time_end
-                    .opt_le(main_state.current_running_time_end)
-                    .unwrap_or(false));
+                assert!(
+                    current_running_time_end
+                        .opt_le(main_state.current_running_time_end)
+                        .unwrap_or(false)
+                );
 
                 // We're properly stopped
                 gst::log!(CAT, obj = pad, "Dropping buffer (stopped)");
@@ -1193,11 +1202,12 @@ impl ToggleRecord {
 
                     gst::log!(CAT, obj = pad, "Clipping to segment {:?}", segment);
 
-                    if let Some(data) = data.clip(&state, &segment) {
-                        Ok(HandleResult::Pass(data))
-                    } else {
-                        gst::warning!(CAT, obj = pad, "Complete buffer clipped!");
-                        Ok(HandleResult::Drop)
+                    match data.clip(&state, &segment) {
+                        Some(data) => Ok(HandleResult::Pass(data)),
+                        _ => {
+                            gst::warning!(CAT, obj = pad, "Complete buffer clipped!");
+                            Ok(HandleResult::Drop)
+                        }
                     }
                 } else {
                     gst::log!(
@@ -1626,9 +1636,7 @@ impl ToggleRecord {
                 let mut state = stream.state.lock();
                 state.eos = true;
 
-                let main_is_eos = main_state
-                    .as_ref()
-                    .map_or(true, |main_state| main_state.eos);
+                let main_is_eos = main_state.as_ref().is_none_or(|main_state| main_state.eos);
                 drop(main_state);
 
                 if main_is_eos {
@@ -1798,12 +1806,9 @@ impl ToggleRecord {
                 let (flags, min, max, align) = new_query.result();
                 q.set(flags, min, max, align);
                 q.add_scheduling_modes(
-                    &new_query
+                    new_query
                         .scheduling_modes()
-                        .iter()
-                        .cloned()
-                        .filter(|m| m != &gst::PadMode::Pull)
-                        .collect::<Vec<_>>(),
+                        .filter(|m| m != &gst::PadMode::Pull),
                 );
                 gst::log!(CAT, obj = pad, "Returning {:?}", q.query_mut());
                 true
