@@ -19,6 +19,7 @@ use std::sync::LazyLock;
 use super::parser::{MccLine, MccParser};
 use crate::line_reader::LineReader;
 use crate::parser_utils::TimeCode;
+use crate::st2038anc_utils::convert_to_st2038_buffer;
 
 static CAT: LazyLock<gst::DebugCategory> = LazyLock::new(|| {
     gst::DebugCategory::new(
@@ -28,6 +29,32 @@ static CAT: LazyLock<gst::DebugCategory> = LazyLock::new(|| {
     )
 });
 
+fn is_st2038() -> bool {
+    std::env::var("GST_MCC_AS_CEA")
+        .map(|val| val != "1")
+        .unwrap_or(true)
+}
+
+fn get_closedcaption_caps(did: u8, sdid: u8, framerate: gst::Fraction) -> (Format, gst::Caps) {
+    match (did, sdid) {
+        (0x61, 0x01) => (
+            Format::Cea708Cdp,
+            gst::Caps::builder("closedcaption/x-cea-708")
+                .field("format", "cdp")
+                .field("framerate", framerate)
+                .build(),
+        ),
+        (0x61, 0x02) => (
+            Format::Cea608,
+            gst::Caps::builder("closedcaption/x-cea-608")
+                .field("format", "s334-1a")
+                .field("framerate", framerate)
+                .build(),
+        ),
+        _ => unreachable!(),
+    }
+}
+
 #[derive(Clone, Copy, Debug, PartialEq, Eq)]
 enum Format {
     Cea708Cdp,
@@ -58,6 +85,7 @@ struct State {
     reader: LineReader<gst::MappedBuffer<gst::buffer::Readable>>,
     parser: MccParser,
     format: Option<Format>,
+    need_caps: bool,
     need_segment: bool,
     pending_events: Vec<gst::Event>,
     last_position: Option<gst::ClockTime>,
@@ -75,6 +103,8 @@ struct State {
     last_raw_line: Vec<u8>,
     replay_last_line: bool,
     need_flush_stop: bool,
+
+    handle_as_st2038: bool,
 }
 
 impl Default for State {
@@ -83,6 +113,7 @@ impl Default for State {
             reader: LineReader::new(),
             parser: MccParser::new(),
             format: None,
+            need_caps: true,
             need_segment: true,
             pending_events: Vec::new(),
             last_position: None,
@@ -96,6 +127,7 @@ impl Default for State {
             last_raw_line: Vec::new(),
             replay_last_line: false,
             need_flush_stop: false,
+            handle_as_st2038: is_st2038(),
         }
     }
 }
@@ -245,7 +277,7 @@ impl State {
     fn create_events(
         &mut self,
         imp: &MccParse,
-        format: Option<Format>,
+        did_sdid: Option<(u8, u8)>,
         framerate: gst::Fraction,
     ) -> Vec<gst::Event> {
         let mut events = Vec::new();
@@ -259,28 +291,28 @@ impl State {
             self.need_flush_stop = false;
         }
 
-        if let Some(pull) = &mut self.pull {
-            if pull.need_stream_start {
-                events.push(gst::event::StreamStart::new(&pull.stream_id));
-                pull.need_stream_start = false;
-            }
+        if let Some(pull) = &mut self.pull
+            && pull.need_stream_start
+        {
+            events.push(gst::event::StreamStart::new(&pull.stream_id));
+            pull.need_stream_start = false;
         }
 
-        if let Some(format) = format {
+        if self.handle_as_st2038 {
+            if self.need_caps {
+                self.need_caps = false;
+                let caps = gst::Caps::builder("meta/x-st-2038")
+                    .field("alignment", "packet")
+                    .field("framerate", framerate)
+                    .build();
+                events.push(gst::event::Caps::new(&caps));
+                gst::info!(CAT, imp = imp, "Caps changed to {:?}", &caps);
+            }
+        } else if let Some((did, sdid)) = did_sdid {
+            let (format, caps) = get_closedcaption_caps(did, sdid, framerate);
+
             if self.format != Some(format) {
                 self.format = Some(format);
-
-                let caps = match format {
-                    Format::Cea708Cdp => gst::Caps::builder("closedcaption/x-cea-708")
-                        .field("format", "cdp")
-                        .field("framerate", framerate)
-                        .build(),
-                    Format::Cea608 => gst::Caps::builder("closedcaption/x-cea-608")
-                        .field("format", "s334-1a")
-                        .field("framerate", framerate)
-                        .build(),
-                };
-
                 events.push(gst::event::Caps::new(&caps));
                 gst::info!(CAT, imp = imp, "Caps changed to {:?}", &caps);
             }
@@ -379,14 +411,24 @@ impl MccParse {
                         continue;
                     }
 
-                    let format = match (data[0], data[1]) {
-                        (0x61, 0x01) => Format::Cea708Cdp,
-                        (0x61, 0x02) => Format::Cea608,
-                        (did, sdid) => {
-                            gst::debug!(CAT, imp = self, "Unknown DID {:x} SDID {:x}", did, sdid);
-                            continue;
+                    let (did, sdid) = (data[0], data[1]);
+                    if !state.handle_as_st2038 {
+                        match (did, sdid) {
+                            (0x61, 0x01) | (0x61, 0x02) => {
+                                // Valid CEA-608/708 codes
+                            }
+                            _ => {
+                                gst::debug!(
+                                    CAT,
+                                    imp = self,
+                                    "Unknown DID {:x} SDID {:x}",
+                                    did,
+                                    sdid
+                                );
+                                continue;
+                            }
                         }
-                    };
+                    }
 
                     let len = data[2];
                     if data.len() < 3 + len as usize {
@@ -400,7 +442,7 @@ impl MccParse {
                         continue;
                     }
 
-                    state = self.handle_line(tc, data, format, state)?;
+                    state = self.handle_line(tc, data, did, sdid, state)?;
                 }
                 Ok(Some(MccLine::Caption(tc, None))) => {
                     assert!(state.seeking);
@@ -494,19 +536,40 @@ impl MccParse {
         &self,
         tc: TimeCode,
         data: Vec<u8>,
-        format: Format,
+        did: u8,
+        sdid: u8,
         mut state: MutexGuard<'_, State>,
     ) -> Result<MutexGuard<'_, State>, gst::FlowError> {
         let (framerate, drop_frame) = parse_timecode_rate(state.timecode_rate)?;
-        let events = state.create_events(self, Some(format), framerate);
+        let events = state.create_events(self, Some((did, sdid)), framerate);
         let timecode = state.handle_timecode(self, framerate, drop_frame, tc)?;
 
         let len = data[2] as usize;
-        let mut buffer = gst::Buffer::from_mut_slice(OffsetVec {
-            vec: data,
-            offset: 3,
-            len,
-        });
+        let mut buffer = if state.handle_as_st2038 {
+            convert_to_st2038_buffer(
+                false,
+                0xFF, /* Unknown/unspecified */
+                0xFF, /* Unknown/unspecified */
+                did,
+                sdid,
+                OffsetVec {
+                    vec: data,
+                    offset: 3,
+                    len,
+                }
+                .as_ref(),
+            )
+            .map_err(|err| {
+                gst::error!(CAT, imp = self, "Failed to convert to ST-2038 {err:?}");
+                gst::FlowError::Error
+            })?
+        } else {
+            gst::Buffer::from_mut_slice(OffsetVec {
+                vec: data,
+                offset: 3,
+                len,
+            })
+        };
 
         state.add_buffer_metadata(self, &mut buffer, &timecode, framerate);
 
@@ -596,26 +659,22 @@ impl MccParse {
         Ok(())
     }
 
-    fn scan_duration(&self) -> Result<Option<ValidVideoTimeCode>, gst::LoggableError> {
+    fn scan_duration(&self) -> Result<Option<ValidVideoTimeCode>, gst::FlowError> {
         gst::debug!(CAT, imp = self, "Scanning duration");
 
         /* First let's query the bytes duration upstream */
         let mut q = gst::query::Duration::new(gst::Format::Bytes);
 
         if !self.sinkpad.peer_query(&mut q) {
-            return Err(gst::loggable_error!(
-                CAT,
-                "Failed to query upstream duration"
-            ));
+            gst::error!(CAT, imp = self, "Failed to query upstream duration");
+            return Err(gst::FlowError::Error);
         }
 
         let size = match q.result() {
             gst::GenericFormattedValue::Bytes(Some(size)) => *size,
             _ => {
-                return Err(gst::loggable_error!(
-                    CAT,
-                    "Failed to query upstream duration"
-                ));
+                gst::error!(CAT, imp = self, "Failed to query upstream duration");
+                return Err(gst::FlowError::Error);
             }
         };
 
@@ -633,11 +692,13 @@ impl MccParse {
                     buffers.push(buffer);
                 }
                 Err(flow) => {
-                    return Err(gst::loggable_error!(
+                    gst::debug!(
                         CAT,
+                        imp = self,
                         "Failed to pull buffer while scanning duration: {:?}",
                         flow
-                    ));
+                    );
+                    return Err(flow);
                 }
             }
 
@@ -645,10 +706,10 @@ impl MccParse {
             let mut parser = MccParser::new_scan_captions();
 
             for buf in buffers.iter().rev() {
-                let buf = buf
-                    .clone()
-                    .into_mapped_buffer_readable()
-                    .map_err(|_| gst::loggable_error!(CAT, "Failed to map buffer readable"))?;
+                let buf = buf.clone().into_mapped_buffer_readable().map_err(|_| {
+                    gst::error!(CAT, imp = self, "Failed to map buffer readable");
+                    gst::FlowError::Error
+                })?;
 
                 reader.push(buf);
             }
@@ -659,7 +720,14 @@ impl MccParse {
                 {
                     let state = self.state.lock().unwrap();
                     let (framerate, drop_frame) = parse_timecode_rate(state.timecode_rate)
-                        .map_err(|_| gst::loggable_error!(CAT, "Failed to parse timecode rate"))?;
+                        .inspect_err(|flow| {
+                            gst::error!(
+                                CAT,
+                                imp = self,
+                                "Failed to parse timecode rate: {:?}",
+                                flow
+                            );
+                        })?;
                     if let Ok(mut timecode) = parse_timecode(framerate, drop_frame, tc) {
                         /* We're looking for the total duration */
                         timecode.increment_frame();
@@ -718,12 +786,12 @@ impl MccParse {
     fn loop_fn(&self) {
         let mut state = self.state.lock().unwrap();
         let State {
-            timecode_rate: ref tc_rate,
+            ref timecode_rate,
             ref mut pull,
             ..
         } = *state;
         let pull = pull.as_mut().unwrap();
-        let scan_tc_rate = tc_rate.is_none() && pull.duration.is_none();
+        let scan_tc_rate = timecode_rate.is_none() && pull.duration.is_none();
         let offset = pull.offset;
 
         pull.offset += 4096;
@@ -762,60 +830,65 @@ impl MccParse {
             }
         };
 
-        match self.handle_buffer(buffer, scan_tc_rate) {
-            Ok(_) => {
-                let tc_rate = self.state.lock().unwrap().timecode_rate;
-                if scan_tc_rate && tc_rate.is_some() {
-                    match self.scan_duration() {
-                        Ok(Some(tc)) => {
-                            let mut state = self.state.lock().unwrap();
-                            let pull = state.pull.as_mut().unwrap();
-                            pull.duration = Some(tc.time_since_daily_jam());
-                        }
-                        Ok(None) => {
-                            let mut state = self.state.lock().unwrap();
-                            let pull = state.pull.as_mut().unwrap();
-                            pull.duration = Some(gst::ClockTime::ZERO);
-                        }
-                        Err(err) => {
-                            err.log();
+        if let Err(flow) = self.handle_buffer(buffer, scan_tc_rate) {
+            match flow {
+                gst::FlowError::Flushing => {
+                    gst::debug!(CAT, imp = self, "Pausing after flow {:?}", flow);
+                }
+                gst::FlowError::Eos => {
+                    self.push_eos();
 
-                            gst::element_imp_error!(
-                                self,
-                                gst::StreamError::Decode,
-                                ["Failed to scan duration"]
-                            );
+                    gst::debug!(CAT, imp = self, "Pausing after flow {:?}", flow);
+                }
+                _ => {
+                    self.push_eos();
 
-                            let _ = self.sinkpad.pause_task();
-                        }
-                    }
+                    gst::error!(CAT, imp = self, "Pausing after flow {:?}", flow);
+
+                    gst::element_imp_error!(
+                        self,
+                        gst::StreamError::Failed,
+                        ["Streaming stopped, reason: {:?}", flow]
+                    );
                 }
             }
-            Err(flow) => {
-                match flow {
-                    gst::FlowError::Flushing => {
-                        gst::debug!(CAT, imp = self, "Pausing after flow {:?}", flow);
-                    }
-                    gst::FlowError::Eos => {
-                        self.push_eos();
 
-                        gst::debug!(CAT, imp = self, "Pausing after flow {:?}", flow);
-                    }
-                    _ => {
-                        self.push_eos();
+            let _ = self.sinkpad.pause_task();
+            return;
+        }
 
-                        gst::error!(CAT, imp = self, "Pausing after flow {:?}", flow);
+        if scan_tc_rate && self.state.lock().unwrap().timecode_rate.is_some() {
+            let duration = match self.scan_duration() {
+                Ok(Some(tc)) => tc.time_since_daily_jam(),
+                Ok(None) => gst::ClockTime::ZERO,
+                Err(gst::FlowError::Flushing) => {
+                    gst::debug!(CAT, imp = self, "Failed to scan duration, reason: flushing");
 
-                        gst::element_imp_error!(
-                            self,
-                            gst::StreamError::Failed,
-                            ["Streaming stopped, reason: {:?}", flow]
-                        );
-                    }
+                    let _ = self.sinkpad.pause_task();
+                    return;
                 }
+                Err(flow) => {
+                    gst::error!(
+                        CAT,
+                        imp = self,
+                        "Failed to scan duration, reason: {:?}",
+                        flow
+                    );
 
-                let _ = self.sinkpad.pause_task();
-            }
+                    gst::element_imp_error!(
+                        self,
+                        gst::StreamError::Decode,
+                        ["Failed to scan duration"]
+                    );
+
+                    let _ = self.sinkpad.pause_task();
+                    return;
+                }
+            };
+
+            let mut state = self.state.lock().unwrap();
+            let pull = state.pull.as_mut().unwrap();
+            pull.duration = Some(duration);
         }
     }
 
@@ -1150,26 +1223,35 @@ impl ElementImpl for MccParse {
 
     fn pad_templates() -> &'static [gst::PadTemplate] {
         static PAD_TEMPLATES: LazyLock<Vec<gst::PadTemplate>> = LazyLock::new(|| {
-            let mut caps = gst::Caps::new_empty();
-            {
-                let caps = caps.get_mut().unwrap();
-                let framerate = gst::FractionRange::new(
-                    gst::Fraction::new(1, i32::MAX),
-                    gst::Fraction::new(i32::MAX, 1),
-                );
+            let caps = if is_st2038() {
+                gst::Caps::builder("meta/x-st-2038")
+                    .field("alignment", "packet")
+                    .build()
+            } else {
+                let mut caps = gst::Caps::new_empty();
+                {
+                    let caps = caps.get_mut().unwrap();
+                    let framerate = gst::FractionRange::new(
+                        gst::Fraction::new(1, i32::MAX),
+                        gst::Fraction::new(i32::MAX, 1),
+                    );
 
-                let s = gst::Structure::builder("closedcaption/x-cea-708")
-                    .field("format", "cdp")
-                    .field("framerate", framerate)
-                    .build();
-                caps.append_structure(s);
+                    let s = gst::Structure::builder("closedcaption/x-cea-708")
+                        .field("format", "cdp")
+                        .field("framerate", framerate)
+                        .build();
+                    caps.append_structure(s);
+
+                    let s = gst::Structure::builder("closedcaption/x-cea-608")
+                        .field("format", "s334-1a")
+                        .field("framerate", framerate)
+                        .build();
+                    caps.append_structure(s);
+                }
+
+                caps
+            };
 
-                let s = gst::Structure::builder("closedcaption/x-cea-608")
-                    .field("format", "s334-1a")
-                    .field("framerate", framerate)
-                    .build();
-                caps.append_structure(s);
-            }
             let src_pad_template = gst::PadTemplate::new(
                 "src",
                 gst::PadDirection::Src,
